import numpy as np
import sys,os
import torch
import torch.nn as nn
import torch.nn.parallel
import torch.backends.cudnn as cudnn
import torch.optim as optim
import torchvision.utils as vutils
import architecture
import Pk_library as PKL

# This routine computes the PDF
def compute_PDF(T_maps, bins_T, mean_T, fout):
    total = T_maps.shape[0]*T_maps.shape[1]*T_maps.shape[2]
    histo = np.histogram(T_maps, bins=bins_T)[0]*1.0/total
    np.savetxt(fout, np.transpose([mean_T, histo]))
    return mean_T, histo

# This routine computes the Pk
def compute_Pk(T_maps, BoxSize, fout):
    Pk = np.zeros((T_maps.shape[0],45), dtype=np.float64)
    for i in range(T_maps.shape[0]):
        Pk_real = PKL.Pk_plane(T_maps[i], BoxSize, MAS='None', threads=1, verbose=False)
        k, Pk[i] = Pk_real.k, Pk_real.Pk
    np.savetxt(fout, np.transpose([k, np.mean(Pk,axis=0), np.std(Pk,axis=0)]))
    return k, np.mean(Pk, axis=0), np.std(Pk, axis=0)

################################### INPUT ############################################
root_in = '/mnt/ceph/users/camels/Results/GAN'
f_maps = 'Images_T.npy'

maps = 15000
grid = 64
bins_T = 300

BATCH_SIZE = 100 
Z_DIM      = 100
G_HIDDEN   = 128
EPOCH_NUM  = 25000
REAL_LABEL = 1
FAKE_LABEL = 0

BoxSize = 25.0*grid/250.0
######################################################################################


# use GPUs if available
GPU = torch.cuda.is_available()
device = torch.device("cuda" if GPU else "cpu")
print('GPU:',GPU)
print('Training on',device)
cudnn.benchmark = True      # May train faster but cost more memory


# do a loop over different models and generate maps; save them to disk
for network_number in np.arange(0,1000,1):

    # get file names and check if they already exists
    fin = '%s/models_64/Net_Gen_%d.pt'%(root_in, network_number)
    fout = '%s/test_64/fake_maps_%d.npy'%(root_in, network_number)
    if os.path.exists(fout):  continue
    if not(os.path.exists(fin)):  continue

    print(network_number)

    # define generator network and load it
    netG = architecture.Generator_64(Z_DIM, G_HIDDEN).to(device)
    netG.load_state_dict(torch.load(fin))

    # define the matrix hosting the fake maps
    fake_maps = np.zeros((maps,64,64), dtype=np.float32)

    batches = maps//BATCH_SIZE
    for i in range(batches):

        start, end = i*BATCH_SIZE, (i+1)*BATCH_SIZE

        # generate points in latent space to generate test images
        z = torch.randn(BATCH_SIZE, Z_DIM, 1, 1, device=device)
        
        with torch.no_grad():
            T_maps = netG(z)
            T_maps = T_maps.squeeze(1)
            fake_maps[start:end,:,:] = T_maps.to('cpu').numpy()
            #vutils.save_image(viz_sample, 'fake_samples_new.png', nrow=16, normalize=True)
            
    # save the maps
    np.save(fout, fake_maps)



################################ SUMMARY STATISTICS ##################################
######################################################################################
# define the variables having the distances
dPDF_min, dPk_min = 1e9, 1e9

# read original maps
T_maps = np.load('%s/%s'%(root_in,f_maps))
T_min = np.min(T_maps); T_min_log = np.log10(T_min)
T_max = np.max(T_maps); T_max_log = np.log10(T_max)
T_maps = T_maps[:,:grid,:grid]
print('%.3e < T < %.3e'%(np.min(T_maps),np.max(T_maps)))

# find the bins in T and their mean values
bins_T = np.logspace(np.log10(T_min), np.log10(T_max), bins_T)
mean_T = 10**(0.5*(np.log10(bins_T[1:]) + np.log10(bins_T[:-1])))

# compute PDF & Pk
f_PDF = '%s/test_64/PDF_T_real_64.txt'%root_in
f_Pk  = '%s/test_64/Pk_T_real_64.txt'%root_in
T_mean, PDF_real     = compute_PDF(T_maps, bins_T, mean_T, f_PDF)
k, Pk_real, dPk_real = compute_Pk(T_maps, BoxSize, f_Pk)

# select the relevant bins and do log10
PDF_real = np.log10(PDF_real[:270]+1.0)
Pk_real  = np.log10(Pk_real[:32])

# do a loop over the different realizations
for network_number in np.arange(0,1000,1):

    # get the name of the inputm map
    fin   = '%s/test_64/fake_maps_%d.npy'%(root_in, network_number)
    f_PDF = '%s/test_64/PDF_T_fake_%d_64.txt'%(root_in, network_number)
    f_Pk  = '%s/test_64/Pk_T_fake_%d_64.txt'%(root_in, network_number)
    if not(os.path.exists(fin)):  continue

    if not(os.path.exists(f_Pk)): 
        # read the maps
        T_maps = np.load(fin)
        T_maps = 10**((T_maps + 1.0)/2.0*(T_max_log - T_min_log) + T_min_log)
        Tmin, Tmax = np.min(T_maps), np.max(T_maps)
        print('%.3e < T < %.3e'%(Tmin,Tmax))

        # compute PDF & Pk
        T_mean, PDF_fake     = compute_PDF(T_maps, bins_T, mean_T, f_PDF)
        k, Pk_fake, dPk_fake = compute_Pk(T_maps, BoxSize, f_Pk)    
    else:
        T_mean, PDF_fake     = np.loadtxt(f_PDF, unpack=True)
        k, Pk_fake, dPk_fake = np.loadtxt(f_Pk,  unpack=True)

    # select the relevant bins and do log10
    PDF_fake = np.log10(PDF_fake[:270]+1.0)
    Pk_fake  = np.log10(Pk_fake[:32])

    dPDF = np.mean((PDF_real - PDF_fake)**2)
    dPk  = np.mean((Pk_real - Pk_fake)**2)
    if dPDF<dPDF_min:
        dPDF_min = dPDF
        print('%d dPDF dPk = %.3e %.3e (minimum dPDF)'%(network_number,dPDF,dPk))
    if dPk<dPk_min:
        dPk_min = dPk
        print('%d dPDF dPk = %.3e %.3e (minimum dPk)'%(network_number,dPDF,dPk))
####################################################################################
####################################################################################



import numpy as np
import sys,os

def compute_errors(x_train, y_train, x_test, y_test):
    y_pred = function(x_train)
    error_train  = np.sqrt(np.mean((y_pred - y_train)**2/y_train**2))
    error_train2 = np.sqrt(np.mean((y_pred - y_train)**2))

    y_pred = function(x_test)
    error_test  = np.sqrt(np.mean((y_pred - y_test)**2/y_test**2))
    error_test2 = np.sqrt(np.mean((y_pred - y_test)**2))
    
    return error_train, error_train2, error_test, error_test2

"""
def function(params): #errors1.txt
    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
    A = Om*0.637**(A3 - np.cos(A1) + np.cos((z+1)*(Om-1)))
    B = np.sin(s8)**((z+1+A1)*np.exp(-np.tan(np.tan(np.tan(Om)))))
    C = np.zeros((A.shape[0],2), dtype=np.float64)
    C[:,0] = A;  C[:,1] = B
    return np.log(np.amin(C,axis=1))
"""

"""
def function(params): #errors2.txt
    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
    A = np.zeros((z.shape[0],2), dtype=np.float32)
    B = np.zeros((z.shape[0],2), dtype=np.float32)
    A[:,0] = (1+z) - 1.75593240370718*np.exp(s8);  A[:,1] = np.log(A3)
    B[:,0] = np.tan(s8);                           B[:,1] = 0.105*(1+z)/Om
    return -A1**0.393 + (0.726*s8)**np.amax(A,axis=1) + \
        np.log(np.amin(B,axis=1)*np.sin(Om))
"""

"""
def function(params): #errors3.txt
    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
    A = np.zeros((z.shape[0],2), dtype=np.float32)
    B = np.zeros((z.shape[0],2), dtype=np.float32)
    C = np.zeros((z.shape[0],2), dtype=np.float32)
    A[:,0] = A3;                     A[:,1] = s8
    B[:,0] = Om**np.amin(A,axis=1);  B[:,1] = Om**(0.212*(1+z-s8))
    C[:,0] = np.amin(B,axis=1);      C[:,1] = 0.159*(1+z)
    return np.log(s8*(z+0.774)**np.log(s8)*np.amin(C,axis=1)/A1**0.387)
"""

#def function(params): #errors4.txt
#    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
#    return 2.016 + 0.1345*(1+z)*A3 + 2.549*s8*np.log(1.0+z) - 0.0399*(1+z)/Om - s8*A3 - 0.6195*(1+z) - 2.94*A1**0.1384

#def function(params): #errors5.txt
#    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
#    return (1+z)*(0.589*Om - 1.715/(1+z)**2 - 0.397/s8) + 0.501**A1 - 0.0226*A1/Om - A3*np.exp(Om - 0.397*(1+z)/s8 - 0.132*A1)

#def function(params): #errors6.txt
#    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
#    return 0.39**A1 + (1+z)*(0.559*Om - 0.365/s8 - 3.57e-3*A1/(Om*s8)) - (1.777 + s8*A3)/(1+z)

#def function(params): #errors7.txt
#    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
#    return (1+z)*(0.54*Om - 0.352/s8 - 4.366e-3*A1/(Om*s8)) + 0.854*0.418**A1 - (1.732 + s8*A3)/(1+z)

#def function(params): #errors8.txt
#    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
#    return 0.692*A1**0.458/((1+z)*A3) - 1.21*A1**0.406 - 1.631 + 2.534*np.log(1+z) - 0.038*(1+z)/Om + (0.36 - 0.425*(1+z))/s8

def function(params): #errors9.txt
    z,Om,s8,A1,A3 = params[:,0], params[:,1], params[:,2], params[:,3], params[:,4]
    return 2.317*np.log(1+z) + 0.696/((1+z)*A3) - 0.0389*(1+z)/Om -0.379*(1+z)/s8 - 1.333 - A1**0.391



################################### INPUT ###########################################
root_in  = '/mnt/ceph/users/camels/Results/neural_nets/params_2_SFRH'
root_out = '/mnt/ceph/users/camels/Results/symbolic_regression'

f_SFRH   = '%s/SFRH_IllustrisTNG.npy'%root_in
f_params = '%s/params_IllustrisTNG.txt'%root_in

training_num = 700
testing_num  = 300

fout = '%s/errors9.txt'%root_out
#####################################################################################

# read SFRH and params; get the redshifts
SFRH   = np.load(f_SFRH);       SFRH = np.log10(SFRH)
params = np.loadtxt(f_params)
z      = np.linspace(0, 7, 100)

##### training set #####
x_train = np.zeros((training_num*100,5), dtype=np.float32)
y_train = np.zeros((training_num*100, ), dtype=np.float32)

# do a loop over the different realizations
count = 0
for i in range(training_num):
    
    # do a loop over the different redshfits
    for j in range(100):
        x_train[count,0] = z[j]
        x_train[count,1] = params[i,0]
        x_train[count,2] = params[i,1]
        x_train[count,3] = params[i,2]
        x_train[count,4] = params[i,4]
        y_train[count]   = SFRH[i,j]
        count += 1
########################

##### testing set #####
x_test = np.zeros((testing_num*100,5), dtype=np.float32)
y_test = np.zeros((testing_num*100, ), dtype=np.float32)

# do a loop over the different realizations
count = 0
for i in range(training_num, training_num + testing_num):
    
    # do a loop over the different redshfits
    for j in range(100):
        x_test[count,0] = z[j]
        x_test[count,1] = params[i,0]
        x_test[count,2] = params[i,1]
        x_test[count,3] = params[i,2]
        x_test[count,4] = params[i,4]
        y_test[count]   = SFRH[i,j]
        count += 1
########################

# do a loop over the different redshifts
f = open(fout, 'w')
for redshift in z:
    indexes = np.where(x_test[:,0]==redshift)[0]
    error_train, error_train2, error_test, error_test2 = \
    compute_errors(x_train[indexes], y_train[indexes], x_test[indexes], y_test[indexes])
    f.write('%.3f %.3f %.5f %.3f %.5f\n'%(redshift, error_train, error_train2, error_test, error_test2))
f.close()

# compute average error
error_train, error_train2, error_test, error_test2 = \
                                compute_errors(x_train, y_train, x_test, y_test)
print('Average train error  = %.3f'%error_train)
print('Average train error2 = %.3f'%error_train2)
print('Average test error   = %.3f'%error_test)
print('Average test error2  = %.3f'%error_test2)



import numpy as np
import sys, os, time
import torch 
import torch.nn as nn
sys.path.append('../')
import data as data
import architecture as architecture

##################################### INPUT ##########################################
# data parameters
root_in      = '/mnt/ceph/users/camels'
root_out     = '/mnt/ceph/users/camels/Results/neural_nets/SFRH_2_params'
sim          = 'IllustrisTNG'
seed         = 1
realizations = 1000
bins_SFRH    = 100

# architecture parameters
h1 = 750
h2 = 750
h3 = 750
h4 = 1000
dropout_rate = 0.0

# training parameters
batch_size = 128
lr         = 1e-5
epochs     = 100000
wd         = 3e-3

# name of output files
name   = '3hd_750_750_750_0.0_3e-3'
fout   = '%s/losses/%s.txt'%(root_out,name)
fmodel = '%s/models/%s.pt'%(root_out,name)
######################################################################################

# use GPUs if available
if torch.cuda.is_available():
    print("CUDA Available")
    device = torch.device('cuda')
else:
    print('CUDA Not Available')
    device = torch.device('cpu')

# define loss function
criterion = nn.MSELoss() 

# get train and validation sets
print('preparing dataset...')
train_loader = data.create_dataset('train', seed, realizations, root_in, bins_SFRH, 
                                   sim, batch_size, root_out)
valid_loader = data.create_dataset('valid', seed, realizations, root_in, bins_SFRH, 
                                   sim, batch_size, root_out)
test_loader  = data.create_dataset('test',  seed, realizations, root_in, bins_SFRH, 
                                   sim, batch_size, root_out)

# define architecture
#model = architecture.model_1hl(bins_SFRH, h1, 6, dropout_rate)
#model = architecture.model_2hl(bins_SFRH, h1, h2, 6, dropout_rate)
model = architecture.model_3hl(bins_SFRH, h1, h2, h3, 6, dropout_rate)
#model = architecture.model_4hl(bins_SFRH, h1, h2, h3, h4, 6, dropout_rate)
model.to(device=device)
network_total_params = sum(p.numel() for p in model.parameters())
print('total number of parameters in the model = %d'%network_total_params)

# define optimizer and scheduler
optimizer = torch.optim.Adam(model.parameters(), lr=lr, betas=(0.5, 0.999), 
                             weight_decay=wd)	
#scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(optimizer, mode='min', 
#                                factor=0.5, patience=25, verbose=True, eps=1e-8)

# load best-model, if it exists
if os.path.exists(fmodel):  
    print('Loading model...')
    model.load_state_dict(torch.load(fmodel))

# get validation loss
print('Computing initial validation loss')
model.eval()
min_valid_loss, points = 0.0, 0
for params_val, SFRH_val in valid_loader:
    with torch.no_grad():
        params_val = params_val.to(device=device)
        SFRH_val   = SFRH_val.to(device=device)
        params_pred = model(SFRH_val)
        min_valid_loss += (criterion(params_pred, params_val).item())*SFRH_val.shape[0]
        points += SFRH_val.shape[0]
min_valid_loss /= points
print('Initial valid loss = %.3e'%min_valid_loss)

# see if results for this model are available
if os.path.exists(fout):  
    dumb = np.loadtxt(fout, unpack=False)
    offset = int(dumb[:,0][-1]+1)
else:   offset = 0

# do a loop over all epochs
start = time.time()
for epoch in range(offset, offset+epochs):
        
    # do training
    train_loss, points = 0.0, 0
    model.train()
    for params_train, SFRH_train in train_loader:
        params_train = params_train.to(device)
        SFRH_train   = SFRH_train.to(device)
        params_pred  = model(SFRH_train)

        loss = criterion(params_pred, params_train)
        train_loss += (loss.item())*SFRH_train.shape[0]
        points += SFRH_train.shape[0]
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()
    train_loss /= points

    # do validation
    valid_loss, points = 0.0, 0
    model.eval()
    for params_val, SFRH_val in valid_loader:
        with torch.no_grad():
            params_val  = params_val.to(device)
            SFRH_val    = SFRH_val.to(device)
            params_pred = model(SFRH_val)
            valid_loss += (criterion(params_pred, params_val).item())*SFRH_val.shape[0]
            points += SFRH_val.shape[0]
    valid_loss /= points

    # do testing
    test_loss, points = 0.0, 0
    model.eval()
    for params_test, SFRH_test in test_loader:
        with torch.no_grad():
            params_test  = params_test.to(device)
            SFRH_test    = SFRH_test.to(device)
            params_pred = model(SFRH_test)
            test_loss += (criterion(params_pred, params_test).item())*SFRH_test.shape[0]
            points += SFRH_test.shape[0]
    test_loss /= points

    #scheduler.step(valid_loss)

    # save model if it is better
    if valid_loss<min_valid_loss:
        print('saving model...')
        torch.save(model.state_dict(), fmodel)
        min_valid_loss = valid_loss
        print('%03d %.3e %.3e %.3e'%(epoch, train_loss, valid_loss, test_loss))
    
    # save losses to file
    f = open(fout, 'a')
    f.write('%d %.5e %.5e %.5e\n'%(epoch, train_loss, valid_loss, test_loss))
    f.close()
    
stop = time.time()
print('Time take (m):', "{:.4f}".format((stop-start)/60.0))



import numpy as np
import sys,os
import torch
import torch.nn as nn
import torch.nn.parallel
import torch.backends.cudnn as cudnn
import torch.optim as optim
import torchvision.utils as vutils
import data
import architecture

################################### INPUT ############################################
root_in  = '/mnt/ceph/users/camels'
root_G   = '/mnt/ceph/users/camels/Results/GAN'
folder_plot = '/mnt/ceph/users/camels/Software/plots/GAN'

seed    = 125
grid    = 64
net_num = 348

# parameters for big images
rows_big    = 6
columns_big = 15

# parameters for interpolation
rows_inter    = 6 #number of interpolations between two images
columns_inter = 6

Z_DIM      = 100
G_HIDDEN   = 128
######################################################################################

z1_index = {0:11, 1:28, 2:89, 3:45, 4:37, 5:3}
z2_index = {0:47, 1:30, 2:24, 3:28, 4:76, 5:66}

# random seed
np.random.seed(seed)
torch.manual_seed(seed)

# use GPUs if available
GPU = torch.cuda.is_available()
device = torch.device("cuda" if GPU else "cpu")
print('GPU:',GPU)
print('Trainin on',device)
cudnn.benchmark = True

# define generator and load best-fit model
netG = architecture.Generator_64(Z_DIM, G_HIDDEN).to(device)
netG.load_state_dict(torch.load('%s/models_64/Net_Gen_%d.pt'%(root_G,net_num)))

######### big images ##########
# get real images images
#train_loader = data.create_train_set(root_G, grid, rows_big*columns_big, verbose=True)
#for data in train_loader:
#    vutils.save_image(data, '%s/real_images.png'%folder_plot, 
#                      nrow=columns_big, normalize=True, range=(-1,1))
#    break

# define the tensor hosting the points in latent space
z = torch.randn(rows_big*columns_big, Z_DIM, 1, 1, device=device)
fake_maps = netG(z)
vutils.save_image(fake_maps, '%s/fake_images.png'%folder_plot, 
                  nrow=columns_big, normalize=True, range=(-1,1))
###############################

######## interpolation ########
# define the tensor hosting the points in latent space
z_inter = np.zeros((rows_inter*columns_inter, Z_DIM, 1, 1), dtype=np.float32)

# define the vector for interpolate points in latent space
t = np.linspace(0, 1, columns_inter)

# do a loop over the different rows (interpolations)
count = 0
for i in range(rows_inter):

    # generate points in latent space to generate test images
    z1 = z[z1_index[i]][:,0,0].cpu() #np.random.randn(Z_DIM)
    z2 = z[z2_index[i]][:,0,0].cpu() #np.random.randn(Z_DIM)

    cos_angle = np.dot(z1,z2)/np.sqrt(np.dot(z1,z1)*np.dot(z2,z2))
    angle = np.arccos(cos_angle)

    for j in range(columns_inter):
        # linear interpolation
        #z[count,:,0,0] = z1 + (z2-z1)*t[j]
        # Slerp interpolation
        z_inter[count,:,0,0] = (np.sin((1-t[j])*angle)*z1 + np.sin(t[j]*angle)*z2)/np.sin(angle)
        count += 1

# make it a torch tensor
z_inter = torch.tensor(z_inter, device=device)
        
# generate maps and save figure
with torch.no_grad():
    fake_maps = netG(z_inter)
    vutils.save_image(fake_maps, '%s/fake_samples_interp.png'%folder_plot, 
                      nrow=columns_inter, normalize=True, range=(-1,1))






import numpy as np
import sys,os
import torch
import torch.nn as nn
import torch.nn.parallel
import torch.backends.cudnn as cudnn
import torch.optim as optim
import torchvision.utils as vutils
import data
import architecture

################################### INPUT ############################################
root_out = '/mnt/ceph/users/camels/Results/autoencoder'
f_images = '/mnt/ceph/users/camels/Results/GAN/Images_T.npy'
#f_images = '/mnt/ceph/users/camels/Results/GAN/Images_T_SIMBA.npy'

seed       = 5 #to split images between training, validation and testing sets
grid       = 64
batch_size = 128 
BN_dim     = 500
hidden     = 32
examples   = 16

f_model = 'model_a_32_wd=1e-9_noise_500dim.pt'
######################################################################################

# use GPUs if available
GPU = torch.cuda.is_available()
device = torch.device("cuda" if GPU else "cpu")
print('GPU:',GPU)
print('Training on',device)
cudnn.benchmark = True      #May train faster but cost more memory

# define loss function
criterion = nn.MSELoss()

# get the data
test_loader  = data.create_dataset('test', seed, f_images, grid, batch_size, 
                                   verbose=True)
test_batches = len(test_loader)

# define the model and load best model
model = architecture.autoencoder_64a(BN_dim, hidden).to(device)
model.load_state_dict(torch.load('%s/models/%s'%(root_out, f_model)))

# compute test loss
test_loss, num_maps = 0.0, 0
model.eval()
for test_maps,idxs in test_loader:
    with torch.no_grad():
        test_maps  = test_maps.to(device)
        recon_maps = model(test_maps)
        loss = criterion(recon_maps, test_maps)
        test_loss += loss.cpu().item()*test_maps.shape[0]
        num_maps  += test_maps.shape[0]
test_loss = test_loss/num_maps
print('Test loss = %.3e'%test_loss)

# get the test maps and make images
for test_maps,idxs in test_loader:
    test_maps = (test_maps[:examples]).to(device)
    break
input_maps = test_maps + torch.randn_like(test_maps, device=device)*0.1
recon_maps = model(input_maps)
vutils.save_image(torch.cat((input_maps,recon_maps,test_maps)), 
                  '%s/images/images_test_IllustrisTNG.png'%root_out,\
                  normalize=True, nrow=examples, range=(-1.0, 1.0))



import numpy as np
import sys,os
import torch
import torch.nn as nn
import torch.nn.parallel
import torch.backends.cudnn as cudnn
import torch.optim as optim
import torchvision.utils as vutils
import data
import architecture

################################### INPUT ############################################
root_out = '/mnt/ceph/users/camels/Results/autoencoder'
f_images = '/mnt/ceph/users/camels/Results/GAN/Images_T.npy'
#f_images = '/mnt/ceph/users/camels/Results/GAN/Images_T_SIMBA.npy'

seed       = 5 #to split images between training, validation and testing sets
grid       = 64
batch_size = 128 
BN_dim     = 500
hidden     = 32

num_images = 5

minimum, maximum = 3.277, 7.507 

f_model = 'model_fiducial_a_32_wd=1e-10_500dim.pt'
#f_model = 'model_a_32_wd=1e-9_noise_500dim.pt'

seed = 12
######################################################################################

# use GPUs if available
GPU = torch.cuda.is_available()
device = torch.device("cuda" if GPU else "cpu")
print('GPU:',GPU)
print('Training on',device)
cudnn.benchmark = True      #May train faster but cost more memory

# define loss function
criterion = nn.MSELoss()

# get the data
data = np.load(f_images)
data = np.log10(data)
data = 2*(data - minimum)/(maximum-minimum) - 1.0

# get the random indexes
indexes = np.arange(data.shape[0])
np.random.seed(seed)
indexes = np.random.choice(indexes, num_images, replace=False)

images = data[indexes]
subimages = np.zeros((16*num_images,64,64), dtype=np.float32)
count = 0
for number in range(num_images):
    for i in range(4):
        if i==3:  i_start, i_end = 186,  250 
        else:     i_start, i_end = i*64, (i+1)*64

        for j in range(4):
            if j==3:  j_start, j_end = 186,  250 
            else:     j_start, j_end = j*64, (j+1)*64
        
            subimages[count] = images[number,i_start:i_end, j_start:j_end]
            count += 1

subimages = torch.tensor(subimages, dtype=torch.float32)
subimages = torch.unsqueeze(subimages, 1)
subimages = subimages.to(device)

# define the model and load best model
model = architecture.autoencoder_64a(BN_dim, hidden).to(device)
model.load_state_dict(torch.load('%s/models/%s'%(root_out, f_model)))

# get autoencoder prediction
model.eval()
with torch.no_grad():
    recon_maps = model(subimages)
loss = criterion(recon_maps, subimages)
print('Test loss = %.3e'%loss.cpu().item())


images2 = torch.zeros((num_images,250,250), dtype=torch.float32)
count = 0
for number in range(num_images):
    for i in range(4):
        if i==3:  i_start, i_end = 186,  250 
        else:     i_start, i_end = i*64, (i+1)*64

        for j in range(4):
            if j==3:  j_start, j_end = 186,  250 
            else:     j_start, j_end = j*64, (j+1)*64
        
            images2[number, i_start:i_end, j_start:j_end] = recon_maps[count,0]
            count += 1

# get the test maps and make images
images  = torch.tensor(images)
images  = torch.unsqueeze(images,1)
images2 = torch.unsqueeze(images2,1)

indexes = np.arange(num_images)

all_images = torch.zeros((num_images*2,1,250,250), dtype=torch.float32)
all_images[2*indexes]   = images
all_images[2*indexes+1] = images2

vutils.save_image(all_images,#torch.cat((images, images2)), 
                  'big_image.png', normalize=True, range=(-1.0, 1.0),
                  nrow=2)#num_images, 



# This script renames the name of the N-body simulation folders
import numpy as np
import sys,os

################################## INPUT ##############################################
root       = '/mnt/ceph/users/camels/Sims'
simulation = 'IllustrisTNG_DM' #'SIMBA_DM'
#######################################################################################


# LH set
for i in range(1000):
    folder_in  = '%s/%s/%s'%(root,simulation,i)
    folder_out = '%s/%s/LH_%d'%(root,simulation,i)

    if os.path.exists(folder_in):
        os.system('mv %s %s'%(folder_in, folder_out))


# CV set
for i in range(27):
    folder_in  = '%s/%s/1505_%s'%(root,simulation,i)
    folder_out = '%s/%s/CV_%d'%(root,simulation,i)

    if os.path.exists(folder_in):
        os.system('mv %s %s'%(folder_in, folder_out))


# 1P set
for i in range(66):
    folder_in  = '%s/%s/%s'%(root,simulation,1500+i)
    folder_out = '%s/%s/1P_%d'%(root,simulation,i)

    if os.path.exists(folder_in):
        os.system('mv %s %s'%(folder_in, folder_out))


# EX set
folder_in  = '%s/%s/1505_EX'%(root,simulation)
folder_out = '%s/%s/EX_0'%(root, simulation)
if os.path.exists(folder_in):
    os.system('mv %s %s'%(folder_in, folder_out))




import torch 
from torch.utils.data.dataset import Dataset
from torch.utils.data import DataLoader
import numpy as np
import sys, os, time


# This class creates the dataset 
class make_dataset():

    def __init__(self, root, grid, verbose=False):

        # read the data
        f_maps = '%s/Images_T.npy'%root
        data = np.load(f_maps)
        unique_maps = data.shape[0]

        # normalize maps
        data = np.log10(data)
        minimum = np.min(data)
        maximum = np.max(data)
        mean = np.mean(data, dtype=np.float64)
        std  = np.std(data, dtype=np.float64)
        #data = (data - mean)/std
        data = 2*(data - minimum)/(maximum-minimum) - 1.0
        if verbose:
            print('%.3f < T(all) < %.3f'%(np.min(data), np.max(data)))            

        # crop maps
        data = data[:,:grid,:grid]

        # define the matrix hosting all data with all rotations/flipping
        data_all = np.zeros((unique_maps*8, data.shape[1], data.shape[2]), 
                            dtype=np.float32)
        
        # do a loop over all rotations (each is 90 deg)
        total_maps = 0
        for rot in [0,1,2,3]:
            data_rot = np.rot90(data, k=rot, axes=(1,2))

            data_all[total_maps:total_maps+unique_maps,:,:] = data_rot
            total_maps += unique_maps

            data_all[total_maps:total_maps+unique_maps,:,:] = np.flip(data_rot, axis=1)
            total_maps += unique_maps
            
        if verbose:
            print('A total of %d maps used'%total_maps)
            print('%.3f < T < %.3f'%(np.min(data), np.max(data)))

        self.size   = data.shape[0]
        self.output = torch.unsqueeze(torch.tensor(data, dtype=torch.float32),1)
        

    def __len__(self):
        return self.size

    def __getitem__(self, idx):
        return self.output[idx]


# This routine creates the training and validation sets
def create_train_set(root, grid, batch_size, verbose=False):
    train_dataset = make_dataset(root, grid, verbose)
    train_loader  = DataLoader(dataset=train_dataset, batch_size=batch_size, 
                               shuffle=True)
    return train_loader





import numpy as np
from gplearn.genetic import SymbolicRegressor
from gplearn.functions import make_function
import sys,os

def _pow(x1,x2):
    with np.errstate(over='ignore'):
        return _protected_exponent(x2*_protected_log(x1))

def _protected_exponent(x):
    with np.errstate(over='ignore'):
        return np.where(x < 30, np.exp(x), 1e15)

def _protected_log(x):
    with np.errstate(over='ignore'):
        return np.where(x > 1e-5, np.log(x), -11.0)

exp = make_function(function=_protected_exponent, name='exp', arity=1)
log = make_function(function=_protected_log,      name='log', arity=1)
pow = make_function(function=_pow,                name='pow', arity=2)

################################### INPUT ###########################################
root = '/mnt/ceph/users/camels/Results/neural_nets/params_2_SFRH'

f_SFRH   = '%s/SFRH_IllustrisTNG.npy'%root
f_params = '%s/params_IllustrisTNG.txt'%root

training_num = 700
testing_num  = 300

# symbolic regressor parameters
population_size       = 2000
generations           = 1000
tournament_size       = 20
function_set          = ('add', 'sub', 'mul', 'div', exp, log, pow, 
                         'cos', 'sin', 'tan', 'max', 'min')
metric                = 'mse'
init_depth            = (2, 10)
n_jobs                = 1
verbose               = 1
parsimony_coefficient = 0.0015
random_state          = None

# evolution parameters
# p_crossover + p_subtree_mutation + p_hoist_mutation + p_point_mutation <= 1.0
p_crossover        = 0.2
p_subtree_mutation = 0.25
p_hoist_mutation   = 0.2
p_point_mutation   = 0.25
p_point_replace    = 0.05

fout = 'results_59.txt'
#####################################################################################

# read SFRH and params
SFRH   = np.load(f_SFRH);       SFRH = np.log10(SFRH)
params = np.loadtxt(f_params)
z      = np.linspace(0, 7, 100)

##### training set #####
x_train = np.zeros((training_num*100,5), dtype=np.float32)
y_train = np.zeros((training_num*100, ), dtype=np.float32)

# do a loop over the different realizations
count = 0
for i in range(training_num):
    
    # do a loop over the different redshfits
    for j in range(100):
        x_train[count,0] = z[j]+1.0
        x_train[count,1] = params[i,0]
        x_train[count,2] = params[i,1]
        x_train[count,3] = params[i,2]
        x_train[count,4] = params[i,4]
        y_train[count]   = SFRH[i,j]
        count += 1
########################

##### testing set #####
x_test = np.zeros((testing_num*100,5), dtype=np.float32)
y_test = np.zeros((testing_num*100, ), dtype=np.float32)

# do a loop over the different realizations
count = 0
for i in range(training_num, training_num + testing_num):
    
    # do a loop over the different redshfits
    for j in range(100):
        x_test[count,0] = z[j]+1.0
        x_test[count,1] = params[i,0]
        x_test[count,2] = params[i,1]
        x_test[count,3] = params[i,2]
        x_test[count,4] = params[i,4]
        y_test[count]   = SFRH[i,j]
        count += 1
########################


# define model and hyperparameters
model = SymbolicRegressor(population_size=population_size, generations=generations,
                          tournament_size=tournament_size, function_set=function_set,
                          metric=metric, init_depth=init_depth,
                          verbose=verbose, parsimony_coefficient=parsimony_coefficient,
                          p_crossover=p_crossover, random_state=random_state,
                          p_subtree_mutation=p_subtree_mutation,
                          p_hoist_mutation=p_hoist_mutation,
                          p_point_mutation=p_point_mutation)

# fit the model
model.fit(x_train, y_train)

# training score
y_pred = model.predict(x_train)
mse_train = np.mean((y_pred - y_train)**2)
error_train = np.sqrt(np.mean((y_pred - y_train)**2/y_train**2))
print('mse train   = %.3e'%mse_train)
print('error train = %.3e'%error_train)

# testing score
y_pred = model.predict(x_test)
mse_test = np.mean((y_pred - y_test)**2)
error_test = np.sqrt(np.mean((y_pred - y_test)**2/y_test**2))
print('mse test = %.3e'%mse_test)
print('error test = %.3e'%error_test)

print(model._program)


# save results to file
f = open(fout,'w')

f.write('root         = %s\n'%root)
f.write('f_SFRH       = %s\n'%f_SFRH)
f.write('f_params     = %s\n'%f_params)
f.write('training_num = %s\n'%training_num)
f.write('testing_num  = %s\n\n'%testing_num)

f.write('population_size       = %s\n'%population_size)
f.write('generations           = %s\n'%generations)
f.write('tournament_size       = %s\n'%tournament_size)
f.write('function_set          = %s\n'%str(function_set))
f.write('metric                = %s\n'%metric)
f.write('init_depth            = %s\n'%str(init_depth))
f.write('n_jobs                = %s\n'%n_jobs)
f.write('verbose               = %s\n'%verbose)
f.write('parsimony_coefficient = %s\n'%parsimony_coefficient)
f.write('random_state          = %s\n\n'%random_state)

f.write('p_crossover        = %s\n'%p_crossover)
f.write('p_subtree_mutation = %s\n'%p_subtree_mutation)
f.write('p_hoist_mutation   = %s\n'%p_hoist_mutation)
f.write('p_point_mutation   = %s\n'%p_point_mutation)
f.write('p_point_replace    = %s\n\n'%p_point_replace)

f.write('mse train   = %.3e\n'%mse_train)
f.write('error train = %.3e\n'%error_train)
f.write('mse test    = %.3e\n'%mse_test)
f.write('error test  = %.3e\n'%error_test)

f.write('%s\n'%model._program)

f.close()



import numpy as np
import sys,os

root = '/mnt/ceph/users/camels/Sims/IllustrisTNG'
fin  = 'latin_hypercube_params.txt'

params = np.loadtxt(fin)

for i in range(1000):
    fout = '%s/LH_%d/CosmoAstro_params.txt'%(root,i)
    if not(os.path.exists(fout)):  
        print(fout)
        f = open(fout, 'w')
        f.write('%.5f %.5f %.5f %.5f %.5f %.5f'\
                %(params[i][0], params[i][1], params[i][2], 
                  params[i][3], params[i][4], params[i][5]))



import numpy as np
import sys,os
import torch
import torch.nn as nn
import torch.nn.parallel
import torch.backends.cudnn as cudnn
import torch.optim as optim
import torchvision.utils as vutils
import data
import architecture

################################### INPUT ############################################
root_in  = '/mnt/ceph/users/camels/Results/autoencoder'
root_out = '/mnt/ceph/users/camels/Software/plots/autoencoder'
f_images = '/mnt/ceph/users/camels/Software/plots/autoencoder/CAMEL.npy'

seed       = 5 #to split images between training, validation and testing sets
grid       = 64
batch_size = 128 
BN_dim     = 500
hidden     = 32

num_images = 5

minimum, maximum = 3.277, 7.507 

f_model = 'model_fiducial_a_32_wd=1e-10_500dim.pt'
#f_model = 'model_a_32_wd=1e-9_noise_500dim.pt'

seed = 12
######################################################################################

# use GPUs if available
GPU = torch.cuda.is_available()
device = torch.device("cuda" if GPU else "cpu")
print('GPU:',GPU)
print('Training on',device)
cudnn.benchmark = True      #May train faster but cost more memory

# define variables needed to compute the error histogram
bins_histo  = np.logspace(-5,-1, 301)
mean_histo  = 10**(0.5*(np.log10(bins_histo[1:]) + np.log10(bins_histo[:-1])))
dhisto      = bins_histo[1:] - bins_histo[:-1]
histo_error = np.zeros(bins_histo.shape[0]-1, dtype=np.float64)

# define loss function
criterion = nn.MSELoss()

# get the data
data = np.load(f_images)
data = np.transpose(data)
data = 2*data/255.0-1.0
print(data.shape)
print(data.dtype)
print(np.min(data), np.max(data))

images = data
subimages = np.zeros((24*26,64,64), dtype=np.float32)
count = 0
for i in range(26):
    i_start, i_end = i*64, (i+1)*64

    for j in range(24):
        j_start, j_end = j*64, (j+1)*64
        
        subimages[count] = images[i_start:i_end, j_start:j_end]
        count += 1

subimages = torch.tensor(subimages, dtype=torch.float32)
subimages = torch.unsqueeze(subimages, 1)
subimages = subimages.to(device)

# define the model and load best model
model = architecture.autoencoder_64a(BN_dim, hidden).to(device)
model.load_state_dict(torch.load('%s/models/%s'%(root_in, f_model)))

# get autoencoder prediction
model.eval()
with torch.no_grad():
    recon_maps = model(subimages)
loss = criterion(recon_maps, subimages)
print('Test loss = %.3e'%loss.cpu().item())

# compute histogram
error = np.mean((recon_maps.cpu().numpy() - subimages.cpu().numpy())**2, axis=(1,2,3))
histo = np.histogram(error, bins=bins_histo)[0]
histo = histo/900.0
np.savetxt('%s/results/Error_CAMEL.txt'%root_in, np.transpose([mean_histo, histo]))

# plots
indexes = np.where(error<1.3e-3)[0]
vutils.save_image(torch.cat((subimages[indexes], recon_maps[indexes])), 
                  '%s/CAMEL_good_recon.png'%root_out, normalize=True, 
                  range=(-1.0, 1.0), nrow=len(indexes))
indexes = np.where(error>4e-3)[0]
vutils.save_image(torch.cat((subimages[indexes], recon_maps[indexes])), 
                  '%s/CAMEL_bad_recon.png'%root_out, normalize=True, 
                  range=(-1.0, 1.0), nrow=len(indexes))


images2 = torch.zeros((26*64,24*64), dtype=torch.float32)
count = 0
for i in range(26):
    i_start, i_end = i*64, (i+1)*64

    for j in range(24):
        j_start, j_end = j*64, (j+1)*64
        
        images2[i_start:i_end, j_start:j_end] = recon_maps[count,0]
        count += 1

# get the test maps and make images
images  = torch.tensor(images, dtype=torch.float32)

print('Average error = %.3e'%torch.mean((images-images2)**2))

# save plots
#vutils.save_image(torch.cat((images, images2)), 'CAMEL_recon.png', 
#                  normalize=True, range=(-1.0, 1.0), nrow=2)
vutils.save_image(images, '%s/CAMEL_original.png'%root_out, 
                  normalize=True, range=(-1.0, 1.0), nrow=1)
vutils.save_image(images2, '%s/CAMEL_reconstructed.png'%root_out, 
                  normalize=True, range=(-1.0, 1.0), nrow=1)



from pylab import *
import numpy as np
from matplotlib.ticker import ScalarFormatter
import matplotlib.gridspec as gridspec
from mpl_toolkits.axes_grid1.inset_locator import mark_inset
from matplotlib.ticker import AutoMinorLocator
from matplotlib.colors import LogNorm
from matplotlib.patches import Ellipse
rcParams["mathtext.fontset"]='cm'

############################### figure ###########################
#fig=figure(figsize=(15,10))     #give dimensions to the figure
##################################################################

################################ INPUT #######################################
#axes range
##############################################################################

############################ subplots ############################
#gs = gridspec.GridSpec(2,1,height_ratios=[5,2])
#ax1=plt.subplot(gs[0])
#ax2=plt.subplot(gs[1])

#make a subplot at a given position and with some given dimensions
#ax2=axes([0.4,0.55,0.25,0.1])

#gs.update(hspace=0.0,wspace=0.4,bottom=0.6,top=1.05)
#subplots_adjust(left=None, bottom=None, right=None, top=None,
#                wspace=0.5, hspace=0.5)

#set minor ticks
#ax1.xaxis.set_minor_locator(AutoMinorLocator(4))
#ax1.yaxis.set_minor_locator(AutoMinorLocator(4))


#ax1.xaxis.set_major_formatter( NullFormatter() )   #unset x label 
#ax1.yaxis.set_major_formatter( NullFormatter() )   #unset y label

# custom xticks 
#ax1.set_xticks([0.25, 0.5, 1.0])
#ax1.set_yticks([0.25, 0.5, 1.0])
#ax1.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter()) #for log 


#ax1.get_yaxis().set_label_coords(-0.2,0.5)  #align y-axis for multiple plots
##################################################################

##################### special behaviour stuff ####################
#to show error missing error bars in log scale
#ax1.set_yscale('log',nonposy='clip')  #set log scale for the y-axis

#set the x-axis in %f format instead of %e
#ax1.xaxis.set_major_formatter(ScalarFormatter()) 

#set size of ticks
#ax1.tick_params(axis='both', which='major', labelsize=10)
#ax1.tick_params(axis='both', which='minor', labelsize=8)

#set the position of the ylabel 
#ax1.yaxis.set_label_coords(-0.2, 0.4)

#set yticks in scientific notation
#ax1.ticklabel_format(axis='y',style='sci',scilimits=(1,4))

#set the x-axis in %f format instead of %e
#formatter = matplotlib.ticker.FormatStrFormatter('$%.2e$') 
#ax1.yaxis.set_major_formatter(formatter) 

#add two legends in the same plot
#ax5 = ax1.twinx()
#ax5.yaxis.set_major_formatter( NullFormatter() )   #unset y label 
#ax5.legend([p1,p2],['0.0 eV','0.3 eV'],loc=3,prop={'size':14},ncol=1)

#set points to show in the yaxis
#ax1.set_yticks([0,1,2])

#highlight a zoomed region
#mark_inset(ax1, ax2, loc1=2, loc2=4, fc="none",edgecolor='purple')
##################################################################

############################ plot type ###########################
#standard plot
#p1,=ax1.plot(x,y,linestyle='-',marker='None')

#error bar plot with the minimum and maximum values of the error bar interval
#p1=ax1.errorbar(r,xi,yerr=[delta_xi_min,delta_xi_max],lw=1,fmt='o',ms=2,
#               elinewidth=1,capsize=5,linestyle='-') 

#filled area
#p1=ax1.fill_between([x_min,x_max],[1.02,1.02],[0.98,0.98],color='k',alpha=0.2)

#hatch area
#ax1.fill([x_min,x_min,x_max,x_max],[y_min,3.0,3.0,y_min],#color='k',
#         hatch='X',fill=False,alpha=0.5)

#scatter plot
#p1=ax1.scatter(k1,Pk1,c='b',edgecolor='none',s=8,marker='*')

#plot with markers
#pl4,=ax1.plot(ke3,Pk3/Pke3,marker='.',markevery=2,c='r',linestyle='None')

#set size of dashed lines
#ax.plot([0, 1], [0, 1], linestyle='--', dashes=(5, 1)) #length of 5, space of 1

#image plot
#cax = ax1.imshow(densities,cmap=get_cmap('jet'),origin='lower',
#           extent=[x_min, x_max, y_min, y_max],
#           #vmin=min_density,vmax=max_density)
#           norm = LogNorm(vmin=min_density,vmax=max_density))
#cbar = fig.colorbar(cax, ax2, ax=ax1, ticks=[-1, 0, 1]) #in ax2 colorbar of ax1
#cbar.set_label(r"$M_{\rm CSF}\/[h^{-1}M_\odot]$",fontsize=14,labelpad=-50)
#cbar.ax.tick_params(labelsize=10)  #to change size of ticks

#make a polygon
#polygon = Rectangle((0.4,50.0), 20.0, 20.0, edgecolor='purple',lw=0.5,
#                    fill=False)
#ax1.add_artist(polygon)
####################################################################

x_min, x_max = 0.3, 50.0
y_min, y_max = 0.0, 1.6


#fig=figure()
fig=figure(figsize=(10,6))     #give dimensions to the figure
ax1=fig.add_subplot(231) 
ax2=fig.add_subplot(232) 
ax3=fig.add_subplot(233) 
ax4=fig.add_subplot(234) 
ax5=fig.add_subplot(235) 
ax6=fig.add_subplot(236) 

subplots_adjust(left=None, bottom=None, right=None, top=None,
                wspace=0.05, hspace=0.05)

for ax in [ax1,ax2,ax3,ax4,ax5,ax6]:
    ax.set_xscale('log')
    #ax1.set_yscale('log')

    ax.set_xlim([x_min,x_max])
    ax.set_ylim([y_min,y_max])

for ax in [ax4,ax5,ax6]:
    ax.set_xlabel(r'$k\/[h\/{\rm Mpc}^{-1}]$',fontsize=18)

for ax in [ax1,ax4]:
    ax.set_ylabel(r'$P_{\rm hydro}(k)/P_{\rm N-body}(k)$',fontsize=18)


for ax in [ax1,ax2,ax3]:
    ax.xaxis.set_major_formatter( NullFormatter() )   #unset x label 

for ax in [ax2,ax3,ax5,ax6]:
    ax.yaxis.set_major_formatter( NullFormatter() )   #unset y label


f_out='Pk_ratio.pdf'

root = '/mnt/ceph/users/camels/Results/Pk_ratio'

# IllustrisTNG z=0
count = 0
for i in range(1000):
    fin = '%s/IllustrisTNG/%d/Pk_ratio_m_z=0.00.txt'%(root,i)
    k, Pk_ratio = np.loadtxt(fin, unpack=True)
    p1,=ax1.plot(k,Pk_ratio,linestyle='-',marker='None',c='k',alpha=0.1)
    count += 1
print('Found %d realizations'%count)

# IllustrisTNG z=2
count = 0
for i in range(1000):
    fin = '%s/IllustrisTNG/%d/Pk_ratio_m_z=2.00.txt'%(root,i)
    k, Pk_ratio = np.loadtxt(fin, unpack=True)
    p1,=ax2.plot(k,Pk_ratio,linestyle='-',marker='None',c='k',alpha=0.1)
    count += 1
print('Found %d realizations'%count)

# IllustrisTNG z=5
count = 0
for i in range(1000):
    fin = '%s/IllustrisTNG/%d/Pk_ratio_m_z=5.00.txt'%(root,i)
    k, Pk_ratio = np.loadtxt(fin, unpack=True)
    p1,=ax3.plot(k,Pk_ratio,linestyle='-',marker='None',c='k',alpha=0.1)
    count += 1
print('Found %d realizations'%count)

# SIMBA z=0
count = 0
for i in range(1000):
    fin = '%s/SIMBA/%d/Pk_ratio_m_z=0.00.txt'%(root,i)
    k, Pk_ratio = np.loadtxt(fin, unpack=True)
    p1,=ax4.plot(k,Pk_ratio,linestyle='-',marker='None',c='k',alpha=0.1)
    count += 1
print('Found %d realizations'%count)

# SIMBA z=2
count = 0
for i in range(1000):
    fin = '%s/SIMBA/%d/Pk_ratio_m_z=2.00.txt'%(root,i)
    k, Pk_ratio = np.loadtxt(fin, unpack=True)
    p1,=ax5.plot(k,Pk_ratio,linestyle='-',marker='None',c='k',alpha=0.1)
    count += 1
print('Found %d realizations'%count)

# SIMBA z=5
count = 0
for i in range(1000):
    fin = '%s/SIMBA/%d/Pk_ratio_m_z=5.00.txt'%(root,i)
    k, Pk_ratio = np.loadtxt(fin, unpack=True)
    p1,=ax6.plot(k,Pk_ratio,linestyle='-',marker='None',c='k',alpha=0.1)
    count += 1
print('Found %d realizations'%count)

BoxSize = 25.0
grid = 256
kN = 2.0*np.pi/BoxSize*(grid/2.0)
for ax in [ax1,ax2,ax3,ax4,ax5,ax6]:
    ax.plot([kN,kN],[y_min,y_max],linestyle='--',marker='None',c='b',alpha=1.0)

#f1 = 'prueba.txt'

#x,y=np.loadtxt(f1,unpack=True) 

#p1,=ax1.plot(x,y,linestyle='-',marker='None')


#place a label in the plot
ax1.text(0.4,1.4, r"${\rm IllustrisTNG\,:\,z=0}$", fontsize=12, color='k')
ax2.text(0.4,1.4, r"${\rm IllustrisTNG\,:\,z=2}$", fontsize=12, color='k')
ax3.text(0.4,1.4, r"${\rm IllustrisTNG\,:\,z=5}$", fontsize=12, color='k')
ax4.text(0.4,1.4, r"${\rm SIMBA\,:\,z=0}$", fontsize=12, color='k')
ax5.text(0.4,1.4, r"${\rm SIMBA\,:\,z=2}$", fontsize=12, color='k')
ax6.text(0.4,1.4, r"${\rm SIMBA\,:\,z=5}$", fontsize=12, color='k')

#legend
#ax1.legend([p1,p2],
#           [r"$z=3$",
#            r"$z=4$"],
#           loc=0,prop={'size':18},ncol=1,frameon=True)
            
            #columnspacing=2,labelspacing=2)




#ax1.set_title(r'$\sum m_\nu=0.0\/{\rm eV}$',position=(0.5,1.02),size=18)
#title('About as simple as it gets, folks')
#suptitle('About as simple as it gets, folks')  #for title with several panels
#grid(True)
#show()
savefig(f_out, bbox_inches='tight')
close(fig)










###############################################################################
#some useful colors:

#'darkseagreen'
#'yellow'
#"hotpink"
#"gold
#"fuchsia"
#"lime"
#"brown"
#"silver"
#"cyan"
#"dodgerblue"
#"darkviolet"
#"magenta"
#"deepskyblue"
#"orchid"
#"aqua"
#"darkorange"
#"coral"
#"lightgreen"
#"salmon"
#"bisque"



import numpy as np
import sys,os,glob


##################################### INPUT ##########################################
root = '/mnt/ceph/users/camels/Results/neural_nets/SFRH_2_params/losses'
######################################################################################

files = glob.glob('%s/*'%root)

# read all files and print the minimum loss
for f in files:
    data = np.loadtxt(f, unpack=False)
    valid_loss = data[:,2]
    index = np.where(valid_loss==np.min(valid_loss))[-1]
    print('%s\n%.3e\n'%(f,valid_loss[index[0]]))



from pylab import *
import numpy as np
from matplotlib.ticker import ScalarFormatter
import matplotlib.gridspec as gridspec
from mpl_toolkits.axes_grid1.inset_locator import mark_inset
from matplotlib.ticker import AutoMinorLocator
from matplotlib.colors import LogNorm
from matplotlib.patches import Ellipse
rcParams["mathtext.fontset"]='cm'

############################### figure ###########################
#fig=figure(figsize=(15,10))     #give dimensions to the figure
##################################################################

################################ INPUT #######################################
#axes range
##############################################################################

############################ subplots ############################
#gs = gridspec.GridSpec(2,1,height_ratios=[5,2])
#ax1=plt.subplot(gs[0])
#ax2=plt.subplot(gs[1])

#make a subplot at a given position and with some given dimensions
#ax2=axes([0.4,0.55,0.25,0.1])

#gs.update(hspace=0.0,wspace=0.4,bottom=0.6,top=1.05)
#subplots_adjust(left=None, bottom=None, right=None, top=None,
#                wspace=0.5, hspace=0.5)

#set minor ticks
#ax1.xaxis.set_minor_locator(AutoMinorLocator(4))
#ax1.yaxis.set_minor_locator(AutoMinorLocator(4))


#ax1.xaxis.set_major_formatter( NullFormatter() )   #unset x label 
#ax1.yaxis.set_major_formatter( NullFormatter() )   #unset y label

# custom xticks 
#ax1.set_xticks([0.25, 0.5, 1.0])
#ax1.set_yticks([0.25, 0.5, 1.0])
#ax1.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter()) #for log 


#ax1.get_yaxis().set_label_coords(-0.2,0.5)  #align y-axis for multiple plots
##################################################################

##################### special behaviour stuff ####################
#to show error missing error bars in log scale
#ax1.set_yscale('log',nonposy='clip')  #set log scale for the y-axis

#set the x-axis in %f format instead of %e
#ax1.xaxis.set_major_formatter(ScalarFormatter()) 

#set size of ticks
#ax1.tick_params(axis='both', which='major', labelsize=10)
#ax1.tick_params(axis='both', which='minor', labelsize=8)

#set the position of the ylabel 
#ax1.yaxis.set_label_coords(-0.2, 0.4)

#set yticks in scientific notation
#ax1.ticklabel_format(axis='y',style='sci',scilimits=(1,4))

#set the x-axis in %f format instead of %e
#formatter = matplotlib.ticker.FormatStrFormatter('$%.2e$') 
#ax1.yaxis.set_major_formatter(formatter) 

#add two legends in the same plot
#ax5 = ax1.twinx()
#ax5.yaxis.set_major_formatter( NullFormatter() )   #unset y label 
#ax5.legend([p1,p2],['0.0 eV','0.3 eV'],loc=3,prop={'size':14},ncol=1)

#set points to show in the yaxis
#ax1.set_yticks([0,1,2])

#highlight a zoomed region
#mark_inset(ax1, ax2, loc1=2, loc2=4, fc="none",edgecolor='purple')
##################################################################

############################ plot type ###########################
#standard plot
#p1,=ax1.plot(x,y,linestyle='-',marker='None')

#error bar plot with the minimum and maximum values of the error bar interval
#p1=ax1.errorbar(r,xi,yerr=[delta_xi_min,delta_xi_max],lw=1,fmt='o',ms=2,
#               elinewidth=1,capsize=5,linestyle='-') 

#filled area
#p1=ax1.fill_between([x_min,x_max],[1.02,1.02],[0.98,0.98],color='k',alpha=0.2)

#hatch area
#ax1.fill([x_min,x_min,x_max,x_max],[y_min,3.0,3.0,y_min],#color='k',
#         hatch='X',fill=False,alpha=0.5)

#scatter plot
#p1=ax1.scatter(k1,Pk1,c='b',edgecolor='none',s=8,marker='*')

#plot with markers
#pl4,=ax1.plot(ke3,Pk3/Pke3,marker='.',markevery=2,c='r',linestyle='None')

#set size of dashed lines
#ax.plot([0, 1], [0, 1], linestyle='--', dashes=(5, 1)) #length of 5, space of 1

#image plot
#cax = ax1.imshow(densities,cmap=get_cmap('jet'),origin='lower',
#           extent=[x_min, x_max, y_min, y_max],
#           #vmin=min_density,vmax=max_density)
#           norm = LogNorm(vmin=min_density,vmax=max_density))
#cbar = fig.colorbar(cax, ax2, ax=ax1, ticks=[-1, 0, 1]) #in ax2 colorbar of ax1
#cbar.set_label(r"$M_{\rm CSF}\/[h^{-1}M_\odot]$",fontsize=14,labelpad=-50)
#cbar.ax.tick_params(labelsize=10)  #to change size of ticks

#make a polygon
#polygon = Rectangle((0.4,50.0), 20.0, 20.0, edgecolor='purple',lw=0.5,
#                    fill=False)
#ax1.add_artist(polygon)
####################################################################

x_min, x_max = 0.0, 25.0
y_min, y_max = 0.0, 25.0


fig  = figure(figsize=(7,12))     #give dimensions to the figure
gs   = gridspec.GridSpec(9,4)
ax1  = plt.subplot(gs[0])
ax2  = plt.subplot(gs[1])
ax3  = plt.subplot(gs[2])
ax4  = plt.subplot(gs[3])
ax5  = plt.subplot(gs[4])
ax6  = plt.subplot(gs[5])
ax7  = plt.subplot(gs[6])
ax8  = plt.subplot(gs[7])
ax9  = plt.subplot(gs[8])
ax10 = plt.subplot(gs[9])
ax11 = plt.subplot(gs[10])
ax12 = plt.subplot(gs[11])
ax13 = plt.subplot(gs[12])
ax14 = plt.subplot(gs[13])
ax15 = plt.subplot(gs[14])
ax16 = plt.subplot(gs[15])
ax17 = plt.subplot(gs[16])
ax18 = plt.subplot(gs[17])
ax19 = plt.subplot(gs[18])
ax20 = plt.subplot(gs[19])
ax21 = plt.subplot(gs[20])
ax22 = plt.subplot(gs[21])
ax23 = plt.subplot(gs[22])
ax24 = plt.subplot(gs[23])
ax25 = plt.subplot(gs[24])
ax26 = plt.subplot(gs[25])
ax27 = plt.subplot(gs[26])
ax28 = plt.subplot(gs[27])
ax29 = plt.subplot(gs[28])
ax30 = plt.subplot(gs[29])
ax31 = plt.subplot(gs[30])
ax32 = plt.subplot(gs[31])
ax33 = plt.subplot(gs[32])
ax34 = plt.subplot(gs[33])
ax35 = plt.subplot(gs[34])
ax36 = plt.subplot(gs[35])

gs.update(hspace=0.07,wspace=0.05,bottom=0.0,top=1.00)

for ax in [ax1,ax2,ax3,ax4,ax5,ax6,ax7,ax8,ax9,ax10,ax11,ax12,ax13,ax14,ax15,
           ax16,ax17,ax18,ax19,ax20,ax21,ax22,ax23,ax24,ax25,ax26,ax27,ax28,
           ax29,ax30,ax31,ax32,ax33,ax34,ax35,ax36]:
    ax.tick_params(
        axis='both',          # changes apply to the x-axis
        which='both',      # both major and minor ticks are affected
        bottom=False,      # ticks along the bottom edge are off
        top=False,         # ticks along the top edge are off
        left=False,
        right=False,
        labelleft=False,
        labelbottom=False) # labels along the bottom edge are off

#ax1.set_xscale('log')
#ax1.set_yscale('log')

#ax1.set_xlim([x_min,x_max])
#ax1.set_ylim([y_min,y_max])

#ax1.set_xlabel(r'$k\/[h\/{\rm Mpc}^{-1}]$',fontsize=18)
#ax1.set_ylabel(r'$P(k)\,[(h^{-1}{\rm Mpc})^3]$',fontsize=18)


root  = '/mnt/ceph/users/camels/Results/images_EX'
f_out = 'images_EX.pdf'
i     = 7 #map index

f1 = '%s/Images_EX_fiducial_T.npy'%root
f2 = '%s/Images_EX_AGN_T.npy'%root
f3 = '%s/Images_EX_SN_T.npy'%root
f4 = '%s/Images_EX_noFB_T.npy'%root
T1, T2, T3, T4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 

f1 = '%s/Images_EX_fiducial_Z.npy'%root
f2 = '%s/Images_EX_AGN_Z.npy'%root
f3 = '%s/Images_EX_SN_Z.npy'%root
f4 = '%s/Images_EX_noFB_Z.npy'%root
Z1, Z2, Z3, Z4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 
Zsun = 0.02;  Z1 = Z1/Zsun;  Z2 = Z2/Zsun;  Z3 = Z3/Zsun;  Z4 = Z4/Zsun

f1 = '%s/Images_EX_fiducial_Mgas.npy'%root
f2 = '%s/Images_EX_AGN_Mgas.npy'%root
f3 = '%s/Images_EX_SN_Mgas.npy'%root
f4 = '%s/Images_EX_noFB_Mgas.npy'%root
Mg1, Mg2, Mg3, Mg4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 

f1 = '%s/Images_EX_fiducial_Mcdm.npy'%root
f2 = '%s/Images_EX_AGN_Mcdm.npy'%root
f3 = '%s/Images_EX_SN_Mcdm.npy'%root
f4 = '%s/Images_EX_noFB_Mcdm.npy'%root
Mc1, Mc2, Mc3, Mc4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 

f1 = '%s/Images_EX_fiducial_Mstar.npy'%root
f2 = '%s/Images_EX_AGN_Mstar.npy'%root
f3 = '%s/Images_EX_SN_Mstar.npy'%root
f4 = '%s/Images_EX_noFB_Mstar.npy'%root
Ms1, Ms2, Ms3, Ms4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 

f1 = '%s/Images_EX_fiducial_Vgas.npy'%root
f2 = '%s/Images_EX_AGN_Vgas.npy'%root
f3 = '%s/Images_EX_SN_Vgas.npy'%root
f4 = '%s/Images_EX_noFB_Vgas.npy'%root
Vg1, Vg2, Vg3, Vg4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 

f1 = '%s/Images_EX_fiducial_HI.npy'%root
f2 = '%s/Images_EX_AGN_HI.npy'%root
f3 = '%s/Images_EX_SN_HI.npy'%root
f4 = '%s/Images_EX_noFB_HI.npy'%root
HI1, HI2, HI3, HI4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 

f1 = '%s/Images_EX_fiducial_ne.npy'%root
f2 = '%s/Images_EX_AGN_ne.npy'%root
f3 = '%s/Images_EX_SN_ne.npy'%root
f4 = '%s/Images_EX_noFB_ne.npy'%root
ne1, ne2, ne3, ne4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 
ne1 *= 1e20;  ne2 *= 1e20;  ne3 *= 1e20;  ne4 *= 1e20

f1 = '%s/Images_EX_fiducial_P.npy'%root
f2 = '%s/Images_EX_AGN_P.npy'%root
f3 = '%s/Images_EX_SN_P.npy'%root
f4 = '%s/Images_EX_noFB_P.npy'%root
P1, P2, P3, P4 = np.load(f1), np.load(f2), np.load(f3), np.load(f4) 

min_T, max_T   = 2e3, 2e7
min_Z, max_Z   = 7e-10/Zsun, 7e-2/Zsun
min_Mg, max_Mg = 2e9, 1e14
min_Mc, max_Mc = 5e9, 1e15
min_Ms, max_Ms = 2e8, 1e15
min_Vg, max_Vg = 50.0, 500
min_HI, max_HI = 1e4, 1e14
min_ne, max_ne = 1e25, 1e33
min_P,  max_P  = 1e0, 1e11

Ms1[np.where(Ms1<min_Ms)] = min_Ms
Ms2[np.where(Ms2<min_Ms)] = min_Ms
Ms3[np.where(Ms3<min_Ms)] = min_Ms
Ms4[np.where(Ms4<min_Ms)] = min_Ms

dy = 0.112

for ax,T in zip([ax1,ax2,ax3,ax4],[T1[i], T2[i], T3[i], T4[i]]):
    cax = ax.imshow(T,cmap=get_cmap('hot'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    norm = LogNorm(vmin=min_T,vmax=max_T))

axa = axes([0.91, 0.896, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax4) #in ax2 colorbar of ax1
cbar.set_label(r"$T_{\rm g}\,[K]$",fontsize=14,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks

for ax,Z in zip([ax5,ax6,ax7,ax8],[Z1[i], Z2[i], Z3[i], Z4[i]]):
    cax = ax.imshow(Z,cmap=get_cmap('cubehelix'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    norm = LogNorm(vmin=min_Z,vmax=max_Z))

axa = axes([0.91, 0.896-dy, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax8) #in ax2 colorbar of ax1
cbar.set_label(r"$Z/Z_\odot$",fontsize=14,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks

for ax,Mg in zip([ax9,ax10,ax11,ax12],[Mg1[i], Mg2[i], Mg3[i], Mg4[i]]):
    cax = ax.imshow(Mg,cmap=get_cmap('jet'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    norm = LogNorm(vmin=min_Mg,vmax=max_Mg))

axa = axes([0.91, 0.896-2*dy, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax12) #in ax2 colorbar of ax1
cbar.set_label(r"$\Sigma_{\rm g}\,[hM_\odot{\rm Mpc}^{-2}]$",fontsize=12,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks

for ax,Ms in zip([ax13,ax14,ax15,ax16],[Ms1[i], Ms2[i], Ms3[i], Ms4[i]]):
    cax = ax.imshow(Ms,cmap=get_cmap('nipy_spectral'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    norm = LogNorm(vmin=min_Ms, vmax=max_Ms))

axa = axes([0.91, 0.896-3*dy, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax12) #in ax2 colorbar of ax1
cbar.set_label(r"$\Sigma_{\rm *}\,[hM_\odot{\rm Mpc}^{-2}]$",fontsize=12,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks

for ax,Vg in zip([ax17,ax18,ax19,ax20],[Vg1[i], Vg2[i], Vg3[i], Vg4[i]]):
    cax = ax.imshow(Vg,cmap=get_cmap('rainbow'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    vmin=min_Vg, vmax=max_Vg)
                    #norm = LogNorm(vmin=min_Vg, vmax=max_Vg))

axa = axes([0.91, 0.896-4*dy, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax12) #in ax2 colorbar of ax1
cbar.set_label(r"$|\vec{V}_{\rm g}|\,[{\rm km/s}]$",fontsize=14,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks

for ax,HI in zip([ax21,ax22,ax23,ax24],[HI1[i], HI2[i], HI3[i], HI4[i]]):
    cax = ax.imshow(HI,cmap=get_cmap('magma'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    norm = LogNorm(vmin=min_HI, vmax=max_HI))

axa = axes([0.91, 0.896-5*dy, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax12) #in ax2 colorbar of ax1
cbar.set_label(r"$\Sigma_{\rm HI}\,[hM_\odot{\rm Mpc}^{-2}]$",fontsize=12,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks

for ax,Mc in zip([ax25,ax26,ax27,ax28],[Mc1[i], Mc2[i], Mc3[i], Mc4[i]]):
    cax = ax.imshow(Mc,cmap=get_cmap('gist_stern'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    norm = LogNorm(vmin=min_Mc, vmax=max_Mc))

axa = axes([0.91, 0.896-6*dy, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax12) #in ax2 colorbar of ax1
cbar.set_label(r"$\Sigma_{\rm DM}\,[hM_\odot{\rm Mpc}^{-2}]$",fontsize=12,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks

for ax,ne in zip([ax29,ax30,ax31,ax32],[ne1[i], ne2[i], ne3[i], ne4[i]]):
    cax = ax.imshow(ne,cmap=get_cmap('gist_earth'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    norm = LogNorm(vmin=min_ne, vmax=max_ne))

axa = axes([0.91, 0.896-7*dy, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax12) #in ax2 colorbar of ax1
cbar.set_label(r"$\Sigma_{\rm e}\,[h{\rm cm}^{-3}{\rm Mpc}^{-1}]$",fontsize=11,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks

for ax,P in zip([ax33,ax34,ax35,ax36],[P1[i], P2[i], P3[i], P4[i]]):
    cax = ax.imshow(P,cmap=get_cmap('terrain'),origin='lower',
                    interpolation='bicubic', extent=[x_min, x_max, y_min, y_max],
                    norm = LogNorm(vmin=min_P, vmax=max_P))

axa = axes([0.91, 0.896-8*dy, 0.015, 0.105])
cbar = fig.colorbar(cax, axa, ax=ax12) #in ax2 colorbar of ax1
cbar.set_label(r"$P_{\rm g}\,[h^2M_\odot{\rm kms}^{-1}{\rm kpc}^{-3}]$",fontsize=10,labelpad=5)
cbar.ax.tick_params(labelsize=8)  #to change size of ticks


#cbar = fig.colorbar(cax, ax2, ax=ax1, ticks=[-1, 0, 1]) #in ax2 colorbar of ax1
#cbar.set_label(r"$M_{\rm CSF}\/[h^{-1}M_\odot]$",fontsize=14,labelpad=-50)
#cbar.ax.tick_params(labelsize=10)  #to change size of ticks

#p1,=ax1.plot(x,y,linestyle='-',marker='None')


#place a label in the plot
#ax1.text(0.2,0.1, r"$z=4.0$", fontsize=22, color='k',transform=ax1.transAxes)

#legend
#ax1.legend([p1,p2],
#           [r"$z=3$",
#            r"$z=4$"],
#           loc=0,prop={'size':18},ncol=1,frameon=True)
            
            #columnspacing=2,labelspacing=2)




#ax1.set_title(r'$\sum m_\nu=0.0\/{\rm eV}$',position=(0.5,1.02),size=18)
#title('About as simple as it gets, folks')
#suptitle('About as simple as it gets, folks')  #for title with several panels
#grid(True)
#show()
savefig(f_out, bbox_inches='tight', dpi=150)
close(fig)










###############################################################################
#some useful colors:

#'darkseagreen'
#'yellow'
#"hotpink"
#"gold
#"fuchsia"
#"lime"
#"brown"
#"silver"
#"cyan"
#"dodgerblue"
#"darkviolet"
#"magenta"
#"deepskyblue"
#"orchid"
#"aqua"
#"darkorange"
#"coral"
#"lightgreen"
#"salmon"
#"bisque"



from pylab import *
import numpy as np
from matplotlib.ticker import ScalarFormatter
import matplotlib.gridspec as gridspec
from mpl_toolkits.axes_grid1.inset_locator import mark_inset
from matplotlib.ticker import AutoMinorLocator
from matplotlib.colors import LogNorm
from matplotlib.patches import Ellipse
rcParams["mathtext.fontset"]='cm'


################################## INPUT #######################################
root  = '/mnt/ceph/users/camels/Results'
f_out = 'test_SIMBA.pdf'

linestyles = ['-', '--', '-.', ':']

# Pk m
f1 = ['%s/Pk/SIMBA/0/Pk_m_z=0.00.txt'%root,
      '%s/Pk/SIMBA/0_clean0/Pk_m_z=0.00.txt'%root,
      '%s/Pk/SIMBA/0_clean1/Pk_m_z=0.00.txt'%root,
      '%s/Pk/SIMBA/0_test/Pk_m_z=0.00.txt'%root]

# ratio Pk m
f2 = ['%s/Pk_ratio/SIMBA/0/Pk_ratio_m_z=0.00.txt'%root,
      '%s/Pk_ratio/SIMBA/0_clean0/Pk_ratio_m_z=0.00.txt'%root,
      '%s/Pk_ratio/SIMBA/0_clean1/Pk_ratio_m_z=0.00.txt'%root,
      '%s/Pk_ratio/SIMBA/0_test/Pk_ratio_m_z=0.00.txt'%root]

# Pk g
f3 = ['%s/Pk/SIMBA/0/Pk_g_z=0.00.txt'%root,
      '%s/Pk/SIMBA/0_clean0/Pk_g_z=0.00.txt'%root,
      '%s/Pk/SIMBA/0_clean1/Pk_g_z=0.00.txt'%root,
      '%s/Pk/SIMBA/0_test/Pk_g_z=0.00.txt'%root]

# HMF
f4 = ['%s/HMF/SIMBA/0/mass_function_1.00e+10_1.00e+14_30_z=0.00.txt'%root,
      '%s/HMF/SIMBA/0_clean0/mass_function_1.00e+10_1.00e+14_30_z=0.00.txt'%root,
      '%s/HMF/SIMBA/0_clean1/mass_function_1.00e+10_1.00e+14_30_z=0.00.txt'%root,
      '%s/HMF/SIMBA/0_test/mass_function_1.00e+10_1.00e+14_30_z=0.00.txt'%root]

# SFRH
f5 = ['%s/SFRH/SIMBA/0/SFRH_0.00_6.00_10000.txt'%root,
      '%s/SFRH/SIMBA/0_clean0/SFRH_0.00_6.00_10000.txt'%root,
      '%s/SFRH/SIMBA/0_clean1/SFRH_0.00_6.00_10000.txt'%root,
      '%s/SFRH/SIMBA/0_test/SFRH_0.00_6.00_10000.txt'%root]
      
# SMF
f6 = ['%s/SMF/SIMBA/0/SMF_1.00e+09_5e+11_10_z=0.00.txt'%root,
      '%s/SMF/SIMBA/0_clean0/SMF_1.00e+09_5e+11_10_z=0.00.txt'%root,
      '%s/SMF/SIMBA/0_clean1/SMF_1.00e+09_5e+11_10_z=0.00.txt'%root,
      '%s/SMF/SIMBA/0_test/SMF_1.00e+09_5e+11_10_z=0.00.txt'%root]

# baryon fraction
f7 = ['%s/baryon_fraction/SIMBA/0/bf_1.00e+10_1.00e+14_30_z=0.00.txt'%root,
      '%s/baryon_fraction/SIMBA/0_clean0/bf_1.00e+10_1.00e+14_30_z=0.00.txt'%root,
      '%s/baryon_fraction/SIMBA/0_clean1/bf_1.00e+10_1.00e+14_30_z=0.00.txt'%root,
      '%s/baryon_fraction/SIMBA/0_test/bf_1.00e+10_1.00e+14_30_z=0.00.txt'%root]

# halos temperature
f8 = ['%s/SO/SIMBA/0/SO_z=0.00.txt'%root,
      '%s/SO/SIMBA/0_clean0/SO_z=0.00.txt'%root,
      '%s/SO/SIMBA/0_clean1/SO_z=0.00.txt'%root,
      '%s/SO/SIMBA/0_test/SO_z=0.00.txt'%root]

# Radii
f9 = ['%s/Radii/SIMBA/0/R_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
      '%s/Radii/SIMBA/0_clean0/R_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
      '%s/Radii/SIMBA/0_clean1/R_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
      '%s/Radii/SIMBA/0_test/R_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root]

# BH masses
f10 = ['%s/BH/SIMBA/0/BH_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/BH/SIMBA/0_clean0/BH_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/BH/SIMBA/0_clean1/BH_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/BH/SIMBA/0_test/BH_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root]

# Vmax
f11 = ['%s/Vmax/SIMBA/0/Vmax_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/Vmax/SIMBA/0_clean0/Vmax_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/Vmax/SIMBA/0_clean1/Vmax_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/Vmax/SIMBA/0_test/Vmax_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root]

# SFR
f12 = ['%s/SFR/SIMBA/0/SFR_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/SFR/SIMBA/0_clean0/SFR_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/SFR/SIMBA/0_clean1/SFR_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root,
       '%s/SFR/SIMBA/0_test/SFR_vs_SM_1.00e+09_5e+11_10_z=0.00.txt'%root]
################################################################################

fig  = figure(figsize=(20,22)) 
gs   = gridspec.GridSpec(4,3)
ax1  = plt.subplot(gs[0])
ax2  = plt.subplot(gs[1])
ax3  = plt.subplot(gs[2])
ax4  = plt.subplot(gs[3])
ax5  = plt.subplot(gs[4])
ax6  = plt.subplot(gs[5])
ax7  = plt.subplot(gs[6])
ax8  = plt.subplot(gs[7])
ax9  = plt.subplot(gs[8])
ax10 = plt.subplot(gs[9])
ax11 = plt.subplot(gs[10])
ax12 = plt.subplot(gs[11])
subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.2, hspace=0.2)

###### Pk m plot ######
ax1.set_xscale('log')
ax1.set_yscale('log')
ax1.set_xlim([0.3,35])
ax1.set_ylim([1e-1,1e3])
ax1.set_xlabel(r'$k\,[h{\rm Mpc}^{-1}]$',fontsize=18)
ax1.set_ylabel(r'$P_{\rm m}(k)\,[h^{-3}{\rm Mpc}^3]$',fontsize=18)

for f,ls in zip(f1,linestyles):
    X,Y = np.loadtxt(f, unpack=True)
    ax1.plot(X,Y,c='k',linestyle=ls)
#######################

###### ratio Pk m plot ######
ax2.set_xscale('log')
ax2.set_xlim([0.3,35])
ax2.set_ylim([0.5,1.05])
ax2.set_xlabel(r'$k\,[h{\rm Mpc}^{-1}]$',fontsize=18)
ax2.set_ylabel(r'$P_{\rm hydro}(k)/P_{\rm Nbody}(k)$',fontsize=18)

for f,ls in zip(f2,linestyles):
    X,Y = np.loadtxt(f, unpack=True)
    ax2.plot(X,Y,c='k',linestyle=ls)
#######################

###### Pk g plot ######
ax3.set_xscale('log')
ax3.set_yscale('log')
ax3.set_xlim([0.3,35])
ax3.set_ylim([1e-2,1e3])
ax3.set_xlabel(r'$k\,[h{\rm Mpc}^{-1}]$',fontsize=18)
ax3.set_ylabel(r'$P_{\rm g}(k)\,[h^{-3}{\rm Mpc}^3]$',fontsize=18)

for f,ls in zip(f3,linestyles):
    X,Y = np.loadtxt(f, unpack=True)
    ax3.plot(X,Y,c='k',linestyle=ls)
#######################

######### HMF #########
ax4.set_xscale('log')
ax4.set_yscale('log')
ax4.set_xlabel(r'$M_{\rm halo}/\Omega_{\rm m}\,[h^{-1}M_\odot]$',fontsize=18)
ax4.set_ylabel(r'${\rm HMF}\,[h^{4}{\rm Mpc}^{-3}M_\odot^{-1}]$',fontsize=18)

for f,ls in zip(f4,linestyles):
    A,X,Y = np.loadtxt(f, unpack=True)
    ax4.plot(X,Y,c='k',linestyle=ls)
#######################

###### SFRH ######
ax5.set_yscale('log')
ax5.set_xlim([0.0,6.0])
ax5.set_ylim([1e-3,1e0])
ax5.set_xlabel(r'$z$',fontsize=18)
ax5.set_ylabel(r'${\rm SFRH}\,[M_\odot{\rm yr}^{-1}{\rm Mpc}^{-3}]$',fontsize=18)

for f,ls in zip(f5,linestyles):
    X,Y = np.loadtxt(f, unpack=True)
    ax5.plot(X,Y,c='k',linestyle=ls)
#######################

###### SMF ######
ax6.set_xscale('log')
ax6.set_yscale('log')
ax6.set_xlabel(r'$z$',fontsize=18)
ax6.set_ylabel(r'${\rm SMF}\,[h^4{\rm Mpc}^{-3}M_\odot^{-1}]$',fontsize=18)

for f,ls in zip(f6,linestyles):
    X,Y = np.loadtxt(f, unpack=True)
    ax6.plot(X,Y,c='k',linestyle=ls)
#######################

###### baryon fraction ######
ax7.set_xscale('log')
ax7.set_xlabel(r'$M_{\rm halo}/\Omega_{\rm m}\,[h^{-1}M_\odot]$',fontsize=18)
ax7.set_ylabel(r'$M_{\rm b}/M_{\rm halo}/(\Omega_{\rm b}/\Omega_{\rm m})$',fontsize=18)

for f,ls in zip(f7,linestyles):
    X,Y = np.loadtxt(f, unpack=True)
    ax7.plot(X,Y,c='k',linestyle=ls)
#######################

###### temperature ######
ax8.set_xscale('log')
ax8.set_yscale('log')
ax8.set_xlabel(r'$M_{\rm halo}/\Omega_{\rm m}\,[h^{-1}M_\odot]$',fontsize=18)
ax8.set_ylabel(r'$T_{\rm halo}\,[K]$',fontsize=18)

for f,ls in zip(f8,linestyles):
    data = np.loadtxt(f)
    #ax8.plot(data[:,0],data[:,9],c='k',linestyle=ls)
#########################

######## Radii ##########
ax9.set_xscale('log')
ax9.set_yscale('log')
ax9.set_xlabel(r'$M_*\,[h^{-1}M_\odot]$',fontsize=18)
ax9.set_ylabel(r'$R_{1/2}\,[h^{-1}{\rm kpc}]$',fontsize=18)

for f,ls in zip(f9,linestyles):
    X,Y,N = np.loadtxt(f, unpack=True)
    ax9.plot(X,Y,c='k',linestyle=ls)
#########################

####### BH masses #######
ax10.set_xscale('log')
ax10.set_yscale('log')
ax10.set_xlabel(r'$M_*\,[h^{-1}M_\odot]$',fontsize=18)
ax10.set_ylabel(r'$M_{\rm black-holes}\,[h^{-1}M_\odot]$',fontsize=18)

for f,ls in zip(f10,linestyles):
    X,Y,N = np.loadtxt(f, unpack=True)
    ax10.plot(X,Y,c='k',linestyle=ls)
#########################

######### Vmax ##########
ax11.set_xscale('log')
ax11.set_yscale('log')
ax11.set_xlabel(r'$M_*\,[h^{-1}M_\odot]$',fontsize=18)
ax11.set_ylabel(r'$V_{\rm max}\,[{\rm km/s}]$',fontsize=18)

for f,ls in zip(f11,linestyles):
    X,Y,N = np.loadtxt(f, unpack=True)
    ax11.plot(X,Y,c='k',linestyle=ls)
#########################

########## SFR ##########
ax12.set_xscale('log')
ax12.set_yscale('log')
ax12.set_xlabel(r'$M_*\,[h^{-1}M_\odot]$',fontsize=18)
ax12.set_ylabel(r'${\rm SFR}\,[M_\odot{\rm yr}^{-1}]$',fontsize=18)

for f,ls in zip(f12,linestyles):
    X,Y,N = np.loadtxt(f, unpack=True)
    ax12.plot(X,Y,c='k',linestyle=ls)
#########################

#ax1.set_title(r'$\sum m_\nu=0.0\/{\rm eV}$',position=(0.5,1.02),size=18)
#title('About as simple as it gets, folks')
#suptitle('About as simple as it gets, folks')  #for title with several panels
#grid(True)
#show()
savefig(f_out, bbox_inches='tight')
close(fig)












import numpy as np
import torch


# This function save to a file the mean, std, min and max of the gradients of the 
# different layers
def save_gradients(f_gradients, model):
  
    f = open(f_gradients, 'w')

    # do a loop over all layers
    count = 0
    for name, param in model.named_parameters():
        if (param.requires_grad) and ("bias" not in name) and (param.grad is not None):
            f.write('%d %s %.5e %.5e %.5e %.5e\n'%(count, name, 
                torch.mean(param.grad).item(), torch.std(param.grad).item(),
                torch.min(param.grad).item(),  torch.max(param.grad).item()))
            count +=1

    f.close()



# This script computes the mean and standard deviation of the different statistics
import numpy as np
import sys,os
import camel_library as CL

#################################### INPUT ########################################
root = '/mnt/ceph/users/camels'

# Power spectrum parameters
z_Pk = 0.0

# Halo mass function parameters
RMmin   = 1e10 #Msun/h
RMmax   = 1e14 #Msun/h
RM_bins = 30
z_HMF   = 0.0

# star-formation rate history parameters
z_min     = 0.0
z_max     = 10.0
sfrh_bins = 10000

# stellar mass function parameters
SMmin   = 1e9  #Msun/h
SMmax   = 5e11 #Msun/h
SM_bins = 10
z_SM    = 0.0

# halo temperature parameters
Mmin_T = 1e12
Mmax_T = 1e14
bins_T = 10
z_T    = 0.0

# galaxy radii parameters
SMmin_R   = 1e9  #Msun/h
SMmax_R   = 5e11 #Msun/h
SM_bins_R = 10
snapnum_R = 33
###################################################################################

# define the array with the realizations
#realizations = np.arange(1000)
numbers = []
for i in range(1000): numbers.append('LH_%s'%i)   #CV set
realizations = np.array(numbers)
sim_set      = 'LH'

#numbers = []
#for i in range(27): numbers.append('CV_%s'%i)   #CV set
#realizations = np.array(numbers)
#sim_set = 'CV'

# do a loop over the two hydro sets
for model in ['SIMBA','IllustrisTNG']:

    """
    # Pk
    root_Pk = '%s/Results/Pk'%root
    first, second = 0, 1
    for suffix in ['m', 'c', 'g', 's', 'bh']:
        name = 'Pk_%s_z=%.2f.txt'%(suffix, z_Pk)
        fout = '%s/%s/mean_%s_%s'%(root_Pk, model, sim_set, name)
        CL.mean_std(root_Pk, model, name, realizations,first,second,fout,verbose=True)

    # Pk_hydro / Pk_Nbody
    root_Pkratio = '%s/Results/Pk_ratio'%root
    first, second = 0, 1
    name = 'Pk_ratio_m_z=%.2f.txt'%z_Pk
    fout = '%s/%s/mean_%s_%s'%(root_Pkratio, model, sim_set, name)
    CL.mean_std(root_Pkratio, model, name, realizations,first,second,fout,verbose=True)

    # HMF
    root_HMF = '%s/Results/HMF'%root
    first, second = 1, 2
    name = 'mass_function_%.2e_%.2e_%d_z=%.2f.txt'%(RMmin,RMmax,RM_bins,z_HMF)
    fout = '%s/%s/mean_%s_%s'%(root_HMF, model, sim_set, name)
    CL.mean_std(root_HMF, model, name, realizations, first, second, fout, verbose=True)

    # baryon fraction FoF
    root_bf = '%s/Results/baryon_fraction'%root
    first, second = 0, 1
    name = 'bf_%.2e_%.2e_%d_z=%.2f.txt'%(RMmin, RMmax, RM_bins, z_HMF)
    fout = '%s/%s/mean_%s_%s'%(root_bf, model, sim_set, name)
    CL.mean_std(root_bf, model, name, realizations, first, second, fout, verbose=True)

    # baryon fraction SO
    root_bf = '%s/Results/baryon_fraction_SO'%root
    first, second = 0, 1
    name = 'bf_SO_%.2e_%.2e_%d_z=%.2f.txt'%(RMmin, RMmax, RM_bins, z_HMF)
    fout = '%s/%s/mean_%s_%s'%(root_bf, model, sim_set, name)
    CL.mean_std(root_bf, model, name, realizations, first, second, fout, verbose=True)

    # gas fraction SO
    root_gf = '%s/Results/baryon_fraction_SO'%root
    first, second = 0, 1
    name = 'gf_SO_%.2e_%.2e_%d_z=%.2f.txt'%(RMmin, RMmax, RM_bins, z_HMF)
    fout = '%s/%s/mean_%s_%s'%(root_gf, model, sim_set, name)
    CL.mean_std(root_gf, model, name, realizations, first, second, fout, verbose=True)

    # SFRH
    root_SFRH = '%s/Results/SFRH'%root
    name = 'SFRH_%.2f_%.2f_%d.txt'%(z_min, z_max, sfrh_bins)
    fout = '%s/%s/mean_%s_%s'%(root_SFRH, model, sim_set, name)
    first, second = 0, 1
    CL.mean_std(root_SFRH, model, name, realizations, first, second, fout,verbose=True)

    # SMF
    root_SMF = '%s/Results/SMF'%root
    name = 'SMF_%.2e_%2.e_%d_z=%.2f.txt'%(SMmin, SMmax, SM_bins, z_SM)
    fout = '%s/%s/mean_%s_%s'%(root_SMF, model, sim_set, name)
    first, second = 0, 1
    CL.mean_std(root_SMF, model, name, realizations, first, second, fout, verbose=True)

    # halos temperature
    root_T = '%s/Results/SO'%root
    name = 'SO_z=%.2f.txt'%(z_T)
    fout = '%s/%s/mean_T_%s_%.2e_%.2e_%d_z=%.2f.txt'%(root_T, model, sim_set, 
                                                      Mmin_T, Mmax_T, bins_T, z_T)
    CL.mean_std_T(root_T, model, name, realizations, Mmin_T, Mmax_T, bins_T, 
                  fout, verbose=False)
    """
    # radii, black hole mass, metallicity & Vmax
    root_R   = '%s/Sims'%root
    name     = 'fof_subhalo_tab_%03d.hdf5'%snapnum_R
    fout_R   = '%s/Results/Radii/%s/mean_R_vs_SM_%s_z=0.00.txt'%(root,model,sim_set)
    fout_BH  = '%s/Results/BH/%s/mean_BH_vs_SM_%s_z=0.00.txt'%(root,model,sim_set)
    fout_SFR = '%s/Results/SFR/%s/mean_SFR_vs_SM_%s_z=0.00.txt'%(root,model,sim_set)
    fout_V   = '%s/Results/Vmax/%s/mean_V_vs_SM_%s_z=0.00.txt'%(root,model,sim_set)
    CL.mean_std_SM(root_R, model, name, realizations, SMmin_R, SMmax_R, SM_bins_R, 
                   fout_R, fout_BH, fout_SFR, fout_V, verbose=False)

# do a loop over the two N-body
#for model in ['SIMBA_DM', 'IllustrisTNG_DM']:




import torch 
import torch.nn as nn
import numpy as np
import sys, os, time


######## 1 hidden layer ##########
# inp ---------> size of input data
# h1 ----------> size of first hidden layer
# out ---------> size of output data
# dr ----------> dropout rate
class model_1hl(nn.Module):
    
    def __init__(self, inp, h1, out, dr):
        super(model_1hl, self).__init__()
        
        self.fc1 = nn.Linear(inp, h1) 
        self.fc2 = nn.Linear(h1,  out)
	
        self.dropout   = nn.Dropout(p=dr)
        self.ReLU      = nn.ReLU()
        self.LeakyReLU = nn.LeakyReLU(0.2)
        
        # initialize the weights of the different layers
        for m in self.modules():
            if isinstance(m, nn.BatchNorm3d) or isinstance(m, nn.BatchNorm1d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 1)
            elif isinstance(m, nn.Conv3d) or isinstance(m, nn.ConvTranspose2d) or \
                 isinstance(m, nn.Linear):
                nn.init.kaiming_normal_(m.weight)
       
    # forward pass
    def forward(self, x):
        out = self.dropout(self.LeakyReLU(self.fc1(x)))
        out = self.fc2(out)         
        return out
##################################

######## 2 hidden layers #########
# inp ---------> size of input data
# h1 ----------> size of first hidden layer
# h2 ----------> size of second hidden layer
# out ---------> size of output data
# dr ----------> dropout rate
class model_2hl(nn.Module):
    
    def __init__(self, inp, h1, h2, out, dr):
        super(model_2hl, self).__init__()
        
        self.fc1 = nn.Linear(inp, h1) 
        self.fc2 = nn.Linear(h1,  h2)
        self.fc3 = nn.Linear(h2,  out)
	
        self.dropout   = nn.Dropout(p=dr)
        self.ReLU      = nn.ReLU()
        self.LeakyReLU = nn.LeakyReLU()
        
        # initialize the weights of the different layers
        for m in self.modules():
            if isinstance(m, nn.BatchNorm3d) or isinstance(m, nn.BatchNorm1d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 1)
            elif isinstance(m, nn.Conv3d) or isinstance(m, nn.ConvTranspose2d) or \
                 isinstance(m, nn.Linear):
                nn.init.kaiming_normal_(m.weight)
       
    # forward pass
    def forward(self, x):
        out = self.dropout(self.LeakyReLU(self.fc1(x)))
        out = self.dropout(self.LeakyReLU(self.fc2(out)))
        out = self.fc3(out)         
        return out
##################################

######## 3 hidden layers #########
# inp ---------> size of input data
# h1 ----------> size of first hidden layer
# h2 ----------> size of second hidden layer
# h3 ----------> size of third  hidden layer
# out ---------> size of output data
# dr ----------> dropout rate
class model_3hl(nn.Module):
    
    def __init__(self, inp, h1, h2, h3, out, dr):
        super(model_3hl, self).__init__()
        
        self.fc1 = nn.Linear(inp, h1) 
        self.fc2 = nn.Linear(h1,  h2)
        self.fc3 = nn.Linear(h2,  h3)
        self.fc4 = nn.Linear(h3,  out)
	
        self.dropout   = nn.Dropout(p=dr)
        self.ReLU      = nn.ReLU()
        self.LeakyReLU = nn.LeakyReLU()
        
        # initialize the weights of the different layers
        for m in self.modules():
            if isinstance(m, nn.BatchNorm3d) or isinstance(m, nn.BatchNorm1d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 1)
            elif isinstance(m, nn.Conv3d) or isinstance(m, nn.ConvTranspose2d) or \
                 isinstance(m, nn.Linear):
                nn.init.kaiming_normal_(m.weight)
       
    # forward pass
    def forward(self, x):
        out = self.dropout(self.LeakyReLU(self.fc1(x)))
        out = self.dropout(self.LeakyReLU(self.fc2(out)))
        out = self.dropout(self.LeakyReLU(self.fc3(out)))
        out = self.fc4(out)         
        return out
##################################

######## 4 hidden layers #########
# inp ---------> size of input data
# h1 ----------> size of first hidden layer
# h2 ----------> size of second hidden layer
# h3 ----------> size of third  hidden layer
# h4 ----------> size of fourth hidden layer
# out ---------> size of output data
# dr ----------> dropout rate
class model_4hl(nn.Module):
    
    def __init__(self, inp, h1, h2, h3, h4, out, dr):
        super(model_4hl, self).__init__()
        
        self.fc1 = nn.Linear(inp, h1) 
        self.fc2 = nn.Linear(h1,  h2)
        self.fc3 = nn.Linear(h2,  h3)
        self.fc4 = nn.Linear(h3,  h4)
        self.fc5 = nn.Linear(h4,  out)
	
        self.dropout   = nn.Dropout(p=dr)
        self.ReLU      = nn.ReLU()
        self.LeakyReLU = nn.LeakyReLU()
        
        # initialize the weights of the different layers
        for m in self.modules():
            if isinstance(m, nn.BatchNorm3d) or isinstance(m, nn.BatchNorm1d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 1)
            elif isinstance(m, nn.Conv3d) or isinstance(m, nn.ConvTranspose2d) or \
                 isinstance(m, nn.Linear):
                nn.init.kaiming_normal_(m.weight)
       
    # forward pass
    def forward(self, x):
        out = self.dropout(self.LeakyReLU(self.fc1(x)))
        out = self.dropout(self.LeakyReLU(self.fc2(out)))
        out = self.dropout(self.LeakyReLU(self.fc3(out)))
        out = self.dropout(self.LeakyReLU(self.fc4(out)))
        out = self.fc5(out)         
        return out
##################################



# architecture of model 1
class model1(nn.Module):
    
    def __init__(self, inp, out1, out2, out3, out4, out5, out6):
        super(model1, self).__init__()
        
        self.fc1 = nn.Linear(inp,  out1) 
        self.fc2 = nn.Linear(out1, out2)
        self.fc3 = nn.Linear(out2, out3)
        self.fc4 = nn.Linear(out3, out4)
        self.fc5 = nn.Linear(out4, out5)
        self.fc6 = nn.Linear(out5, out6)
	
        self.dropout   = nn.Dropout(p=0.5)
        self.ReLU      = nn.ReLU()
        self.LeakyReLU = nn.LeakyReLU()
        
        # initialize the weights of the different layers
        for m in self.modules():
            if isinstance(m, nn.BatchNorm3d) or isinstance(m, nn.BatchNorm1d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 1)
            elif isinstance(m, nn.Conv3d) or isinstance(m, nn.ConvTranspose2d) or \
                 isinstance(m, nn.Linear):
                nn.init.kaiming_normal_(m.weight)
       
    # forward pass
    def forward(self, x):
        out = self.dropout(self.LeakyReLU(self.fc1(x)))
        out = self.dropout(self.LeakyReLU(self.fc2(out)))
        out = self.dropout(self.LeakyReLU(self.fc3(out)))
        out = self.dropout(self.LeakyReLU(self.fc4(out)))
        out = self.dropout(self.LeakyReLU(self.fc5(out)))
        out = self.fc6(out)         
        return out


# architecture of model 2
class model2(nn.Module):
    
    def __init__(self, inp, out1, out2, out3):
        super(model2, self).__init__()
        
        self.fc1 = nn.Linear(inp,  out1) 
        self.fc2 = nn.Linear(out1, out2)
        self.fc3 = nn.Linear(out2, out3)
	
        self.dropout   = nn.Dropout(p=0.5)
        self.ReLU      = nn.ReLU()
        self.LeakyReLU = nn.LeakyReLU()
        
        # initialize the weights of the different layers
        for m in self.modules():
            if isinstance(m, nn.BatchNorm3d) or isinstance(m, nn.BatchNorm1d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 1)
            elif isinstance(m, nn.Conv3d) or isinstance(m, nn.ConvTranspose2d) or \
                 isinstance(m, nn.Linear):
                nn.init.kaiming_normal_(m.weight)
       
    # forward pass
    def forward(self, x):
        out = self.dropout(self.LeakyReLU(self.fc1(x)))
        out = self.dropout(self.LeakyReLU(self.fc2(out)))
        out = self.fc3(out)         
        return out


# architecture of model 3
class model3(nn.Module):
    
    def __init__(self, inp, out1, out2):
        super(model3, self).__init__()
        
        self.fc1 = nn.Linear(inp,  out1) 
        self.fc2 = nn.Linear(out1, out2)
	
        self.dropout   = nn.Dropout(p=0.5)
        self.ReLU      = nn.ReLU()
        self.LeakyReLU = nn.LeakyReLU()
        
        # initialize the weights of the different layers
        for m in self.modules():
            if isinstance(m, nn.BatchNorm3d) or isinstance(m, nn.BatchNorm1d):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 1)
            elif isinstance(m, nn.Conv3d) or isinstance(m, nn.ConvTranspose2d) or \
                 isinstance(m, nn.Linear):
                nn.init.kaiming_normal_(m.weight)
       
    # forward pass
    def forward(self, x):
        out = self.dropout(self.LeakyReLU(self.fc1(x)))
        out = self.fc2(out)         
        return out





import numpy as np
import sys,os

####################################### INPUT ###########################################
nodes_per_sim = 1
start         = 500     #first realization to do
end           = 1000   #last  realization do do
#########################################################################################

#numbers = np.arange(1500, 1522, 1)
numbers = ['1505_5',  '1505_6',  '1505_7',  '1505_8',  '1505_9', 
           '1505_10', '1505_11', '1505_12', '1505_13', '1505_14',
           '1505_15', '1505_16', '1505_17', '1505_18', '1505_19', 
           '1505_20', '1505_21', '1505_22', '1505_23', '1505_24',
           '1505_25', '1505_26', '1505_EX']

# do a loop over the different jobs
for i in numbers: #xrange(start,end,1):
    
    script="""#!/bin/bash
#SBATCH --job-name=%s
#SBATCH -o OUTPUT.o%s
#SBATCH -e OUTPUT.e%s  
#SBATCH --partition=general
#####SBATCH --partition=preempt
#SBATCH --nodes=%d
#SBATCH --ntasks-per-node=48
#SBATCH --export=ALL
#SBATCH -t 1-00:00    

module purge
module load slurm
module load gcc
module load openmpi
module load lib/fftw2/2.1.5-openmpi1
module load lib/gsl
module load lib/hdf5

LPT_code="/simons/scratch/fvillaescusa/CAMELS/Codes/2lpt/2LPTic"
G3="/simons/scratch/fvillaescusa/CAMELS/Codes/Gadget_III/P-Gadget3"
root="/simons/scratch/fvillaescusa/CAMELS/Sims/SIMBA_DM/"

folder=$root"%s"
cd $folder

if [ ! -f snap_033.hdf5 ]; then
    if [ ! -f restartfiles/restart.0 ]; then
        cd ICs
        mpirun -n 8 $LPT_code 2LPT.param >> logIC
        cd ..
        mpirun -n %d $G3 G3.param >> logfile    
    else
        mpirun -n %d $G3 G3.param 1 >> logfile    
    fi
fi
    """%(i, i, i, nodes_per_sim, i, nodes_per_sim*48, nodes_per_sim*48)

    # write the submission script inside each folder
    f = open('/simons/scratch/fvillaescusa/CAMELS/Sims/SIMBA_DM/%s/script.sh'%i, 'w')
    f.write(script);  f.close()
    os.chdir('/simons/scratch/fvillaescusa/CAMELS/Sims/SIMBA_DM/%s/'%i)

    os.system('sbatch script.sh')



import numpy as np
import sys,os
import torch
import torch.nn as nn
import torch.nn.parallel
import torch.backends.cudnn as cudnn
import torch.optim as optim
import torchvision.utils as vutils
import data
import architecture

################################### INPUT ############################################
root_out   = '/mnt/ceph/users/camels/Results/autoencoder'
seed       = 5 #to split images between training, validation and testing sets
grid       = 64
batch_size = 128 
BN_dim     = 500
hidden     = 32
examples   = 16
f_model    = 'model_a_32_wd=1e-9_noise_500dim.pt'
sim        = 'IllustrisTNG'
mode       = 'test'  #'train', 'valid', 'test', 'all'
######################################################################################

# get the name of the file containing the images
f_images = '/mnt/ceph/users/camels/Results/autoencoder/Images_T_extreme_0.npy'
f_images = '/mnt/ceph/users/camels/Results/autoencoder/Images_T_extremestellar_0.npy'

# use GPUs if available
GPU = torch.cuda.is_available()
device = torch.device("cuda" if GPU else "cpu")
print('GPU:',GPU)
print('Training on',device)
cudnn.benchmark = True      #May train faster but cost more memory

# define loss function
criterion = nn.MSELoss()

# get the data & crop maps
data = np.load(f_images)
data = np.log10(data)
minimum, maximum = 3.182, 7.948  #np.min(data), np.max(data)
data = 2*(data - minimum)/(maximum-minimum) - 1.0
print('%.3f < T(all) < %.3f'%(np.min(data), np.max(data))) 
#data = data[:,:grid,:grid]
maps = np.zeros((15*12*12, 64, 64), dtype=np.float32)
count = 0 
for map_num in range(15):
    for i in range(12):
        for j in range(12):
            maps[count] = data[map_num, 15*i:15*i+64, 15*j:15*j+64]
            count += 1
test_maps = torch.tensor(maps, dtype=torch.float32)
test_maps = test_maps.unsqueeze(1)

# define the model and load best model
model = architecture.autoencoder_64a(BN_dim, hidden).to(device)
model.load_state_dict(torch.load('%s/models/%s'%(root_out, f_model)))

# define variables needed to compute the error histogram
#bins_histo  = np.logspace(-5,-1, 501)
#mean_histo  = 10**(0.5*(np.log10(bins_histo[1:]) + np.log10(bins_histo[:-1])))
#dhisto      = bins_histo[1:] - bins_histo[:-1]
#histo_error = np.zeros(bins_histo.shape[0]-1, dtype=np.float64)

# compute test loss
test_loss, maps, batches = 0.0, 0, 0
min_error, mean_error, max_error = 1e9, 0.0, 0.0
model.eval()
with torch.no_grad():

    # get input, output maps and loss
    test_maps = test_maps.to(device)
    recon_maps = model(test_maps)
    loss = criterion(recon_maps, test_maps)

    # get cumulative loss and number of maps
    test_loss += loss.cpu().item()
    batches += 1
    maps += test_maps.shape[0]

    """
    # save some images
    if test_maps.shape[0]>examples:
        vutils.save_image(torch.cat((test_maps[:examples],recon_maps[:examples])), 
                          '%s/images/images_%s_%s.png'%(root_out,sim,mode),\
                          normalize=True, nrow=examples, range=(-1.0, 1.0))
    """

    # compute reconstruction error for each map
    error = ((test_maps - recon_maps)**2).cpu().numpy()
    error = np.mean(error, axis=(1,2,3))
    mean_error += np.sum(error)
    #histo_error += np.histogram(error, bins_histo)[0]

    # identify the larger reconstruction error and make image
    index = np.where(error==np.max(error))[0]
    if error[index]>max_error:
        max_error = error[index]
        print('max error = %.3e'%(max_error))
        vutils.save_image(torch.cat([test_maps[index],recon_maps[index]]), 
                          '%s/images/images_outlier_extreme_0.png'%(root_out),\
                          normalize=True, nrow=1, range=(-1.0, 1.0))

    # identify the smaller reconstruction error
    index = np.where(error==np.min(error))[0]
    if error[index]<min_error:
        min_error = error[index]
        print('min error = %.3e'%(min_error))

# finale verbose
print('\nTest loss  = %.3e'%(test_loss/batches))
print('Min = %.3e  :  Max = %.3e'%(min_error, max_error))
print('Mean error = %.3e'%(mean_error/maps))
print('Number of maps = %d'%maps)

# histogram
#histo_error = histo_error/maps/dhisto
#print('Integral histo = %.3e'%(np.sum(histo_error*dhisto)))
#np.savetxt('Errors_%s_%s_500dim.txt'%(sim,mode), np.transpose([mean_histo, histo_error]))



import numpy as np
import sys,os
import torch
import torch.nn as nn
import torch.nn.parallel
import torch.backends.cudnn as cudnn
import torch.optim as optim
import torchvision.utils as vutils
import data
import architecture

################################### INPUT ############################################
root_in  = '/mnt/ceph/users/camels/Results/GAN'
root_out = '/mnt/ceph/users/camels/Results/GAN'

lr_D = 1e-5
lr_G = 1e-4
seed = 1   #mainly for the generated images
grid = 64

BATCH_SIZE = 128 
Z_DIM      = 100
G_HIDDEN   = 128
D_HIDDEN   = 80
EPOCH_NUM  = 1000
REAL_LABEL = 1
FAKE_LABEL = 0
######################################################################################

# use GPUs if available
GPU = torch.cuda.is_available()
device = torch.device("cuda" if GPU else "cpu")
print('GPU:',GPU)
print('Training on',device)
cudnn.benchmark = True      # May train faster but cost more memory

# initialize random number generator
if seed is None:  seed = np.random.randint(1, 10000)
np.random.seed(seed)
torch.manual_seed(seed)
if GPU:  torch.cuda.manual_seed(seed)

# generate points in latent space to generate test images
viz_noise = torch.randn(BATCH_SIZE, Z_DIM, 1, 1, device=device)

# define loss function
criterion = nn.BCELoss()

# get the data
train_loader = data.create_train_set(root_in, grid, BATCH_SIZE, verbose=True)

# define generator and discriminator networks
netG = architecture.Generator_64(Z_DIM, G_HIDDEN).to(device)
netG.apply(architecture.weights_init)
netD = architecture.Discriminator_64(D_HIDDEN).to(device)
netD.apply(architecture.weights_init)

# load best-models, if they exists
netG.load_state_dict(torch.load('%s/models_64/Net_Gen_808.pt'%root_out))
netD.load_state_dict(torch.load('%s/models_64/Net_Dis_808.pt'%root_out))

# define the optimizers
optimizerD = optim.Adam(netD.parameters(), lr=lr_D, betas=(0.5, 0.999))
#optimizerD = optim.SGD(netD.parameters(), lr=lr_D, momentum=0.9)
optimizerG = optim.Adam(netG.parameters(), lr=lr_G, betas=(0.5, 0.999))


# do a loop over all the epochs
for epoch in range(808,2000):
    
    # do a loop over all maps
    for i,true_maps in enumerate(train_loader):

        # find the number of maps in this batch
        num_maps = true_maps.shape[0]

        true_maps = true_maps.to(device)
        real_label = torch.full((num_maps,), REAL_LABEL, device=device)
        fake_label = torch.full((num_maps,), FAKE_LABEL, device=device)

        if epoch==0 and i==0:
            vutils.save_image(true_maps, '%s/images_64/real_samples.png'%root_out, 
                              normalize=True, nrow=16, range=(-1.0,1.0))

        # Update D with real data
        netD.zero_grad()
        y_real = netD(true_maps)
        loss_D_real = criterion(y_real, real_label)
        loss_D_real.backward()

        # Update D with fake data
        z_noise = torch.randn(num_maps, Z_DIM, 1, 1, device=device)
        fake_maps = netG(z_noise)
        y_fake = netD(fake_maps.detach())
        loss_D_fake = criterion(y_fake, fake_label)
        loss_D_fake.backward()
        optimizerD.step()

        # Update G with fake data
        netG.zero_grad()
        y_fake_r = netD(fake_maps)
        loss_G = criterion(y_fake_r, real_label)
        loss_G.backward()
        optimizerG.step()

    # print info about losses and save them to file
    print('Epoch {} loss_D_real: {:.4f} loss_D_fake: {:.4f} loss_G: {:.4f}'\
          .format(epoch, loss_D_real.mean().item(), loss_D_fake.mean().item(),
                  loss_G.mean().item()))
    f = open('%s/losses/loss_64.txt'%root_out, 'a')
    f.write('%d %.3e %.3e %.3e\n'%(epoch,loss_D_real.mean().item(),
                                   loss_D_fake.mean().item(), loss_G.mean().item()))
    f.close()
        
    # save images from generator to file
    with torch.no_grad():
        viz_sample = netG(viz_noise)[:49]
        vutils.save_image(viz_sample, '%s/images_64/fake_samples_%d.png'\
                          %(root_out,epoch), normalize=True, nrow=7, range=(-1.0, 1.0))

    # save networks
    torch.save(netG.state_dict(), '%s/models_64/Net_Gen_%d.pt'%(root_out,epoch))
    torch.save(netD.state_dict(), '%s/models_64/Net_Dis_%d.pt'%(root_out,epoch))




# This script creates images from the extreme IllustrisTNG simulation
import numpy as np
import sys,os,h5py
import MAS_library as MASL
sys.path.append('../analysis')
import camel_library as CL
import torch
import torchvision.utils as vutils

#################################### INPUT ##########################################
root_in       = '/mnt/ceph/users/camels'
root_out      = '/mnt/ceph/users/camels/Results/images_EX'
sim           = 'IllustrisTNG' #'IllustrisTNG'
snapnum       = 33
realizations  = 1000
TREECOOL_file = '/mnt/ceph/users/camels/Software/TREECOOL'

grid         = 250
splits       = 5

# parameters of the density field routine
periodic = True
x_min, y_min = 0.0, 0.0

# name of snapshot and output file
snapshots = ['%s/Sims/IllustrisTNG/fiducial_0/snap_%03d.hdf5'%(root_in, snapnum),
             '%s/Sims/IllustrisTNG/extreme_0/snap_%03d.hdf5'%(root_in,snapnum),
             '%s/Sims/IllustrisTNG/extremestellar_0/snap_%03d.hdf5'%(root_in,snapnum),
             '%s/Sims/IllustrisTNG/noFB_0/snap_%03d.hdf5'%(root_in,snapnum)]

fouts = ['%s/Images_EX_fiducial'%root_out,
         '%s/Images_EX_AGN'%root_out,
         '%s/Images_EX_SN'%root_out,
         '%s/Images_EX_noFB'%root_out] 
#####################################################################################

# do a loop over the different simulations
for snapshot,fout in zip(snapshots, fouts):

    # read the gas positions, radii and temperature of gas particles
    if not(os.path.exists(snapshot)):  raise Exception('snapshot does not exists')
    f = h5py.File(snapshot, 'r')
    BoxSize = f['Header'].attrs[u'BoxSize']/1e3  #Mpc/h
    redshift = f['Header'].attrs[u'Redshift'];  scale_factor = 1.0/(1.0+redshift)
    Masses  = f['Header'].attrs[u'MassTable'][:]*1e10
    pos_g   = f['PartType0/Coordinates'][:]/1e3  #Mpc/h
    pos_c   = f['PartType1/Coordinates'][:]/1e3  #Mpc/h
    pos_s   = f['PartType4/Coordinates'][:]/1e3  #Mpc/h
    vel_g   = f['/PartType0/Velocities'][:]/np.sqrt(1.0+redshift)
    vel_g   = np.sqrt(vel_g[:,0]**2 + vel_g[:,1]**2 + vel_g[:,2]**2)
    Mg      = f['PartType0/Masses'][:]*1e10      #Msun/h
    Mc      = np.ones(pos_c.shape[0], dtype=np.float32)*Masses[1] #Msun/h
    Ms      = f['PartType4/Masses'][:]*1e10      #Msun/h
    Rg      = f['PartType0/SubfindHsml'][:]/1e3  #Mpc/h
    Rc      = f['PartType1/SubfindHsml'][:]/1e3  #Mpc/h
    Rs      = f['PartType4/SubfindHsml'][:]/1e3  #Mpc/h
    Z       = f['/PartType0/GFM_Metallicity'][:] #metallicity
    f.close()
    HI = CL.HI_mass(snapshot, TREECOOL_file)
    T  = CL.temperature(snapshot)
    P  = CL.pressure(snapshot)         #Msun*(km/s)^2/kpc^3
    ne = CL.electron_density(snapshot) #1e20 electrons*h^2/Mpc^3

    # define the matrix hosting all the maps
    maps_T  = np.zeros((splits*3, grid, grid), dtype=np.float32)
    maps_Z  = np.zeros((splits*3, grid, grid), dtype=np.float32)
    maps_Mg = np.zeros((splits*3, grid, grid), dtype=np.float32)
    maps_Mc = np.zeros((splits*3, grid, grid), dtype=np.float32)
    maps_Ms = np.zeros((splits*3, grid, grid), dtype=np.float32)
    maps_Vg = np.zeros((splits*3, grid, grid), dtype=np.float32)
    maps_HI = np.zeros((splits*3, grid, grid), dtype=np.float32)
    maps_P  = np.zeros((splits*3, grid, grid), dtype=np.float32)
    maps_ne = np.zeros((splits*3, grid, grid), dtype=np.float32)

    # do a loop over the three different axes
    count = 0
    for axis in [0,1,2]:

        axis_x, axis_y = (axis+1)%3, (axis+2)%3 

        # do a loop over the different slices of each axis
        for j in range(splits):

            # get the index of the map
            count = splits*axis + j

            # find the range in the slice
            minimum, maximum = j*BoxSize/splits, (j+1)*BoxSize/splits

            # select the particles in the considered slice
            indexes_g = np.where((pos_g[:,axis]>=minimum) & (pos_g[:,axis]<maximum))[0] 
            indexes_c = np.where((pos_c[:,axis]>=minimum) & (pos_c[:,axis]<maximum))[0] 
            indexes_s = np.where((pos_s[:,axis]>=minimum) & (pos_s[:,axis]<maximum))[0] 
            pos_g_slice = pos_g[indexes_g].astype(np.float32)
            pos_c_slice = pos_c[indexes_c].astype(np.float32)
            pos_s_slice = pos_s[indexes_s].astype(np.float32)
            Rg_slice    = Rg[indexes_g]
            Rc_slice    = Rc[indexes_c]
            Rs_slice    = Rs[indexes_s]
            Mg_slice    = Mg[indexes_g]
            Mc_slice    = Mc[indexes_c]
            Ms_slice    = Ms[indexes_s]
            Vg_slice    = vel_g[indexes_g]
            T_slice     = T[indexes_g]
            Z_slice     = Z[indexes_g]
            HI_slice    = HI[indexes_g]
            P_slice     = P[indexes_g]
            ne_slice    = ne[indexes_g]

            # project particle mass*temperatures into a 2D maps
            TM = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(TM, pos_g_slice, T_slice*Mg_slice, Rg_slice, 
                    x_min, y_min, axis_x, axis_y, BoxSize, periodic, verbose=False)

            # project particle mass*metallicity into a 2D maps
            ZM = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(ZM, pos_g_slice, Z_slice*Mg_slice, Rg_slice, 
                        x_min, y_min, axis_x, axis_y, BoxSize, periodic, verbose=False)

            # project particle pressure*mass into a 2D maps
            PM = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(PM, pos_g_slice, P_slice*Mg_slice, Rg_slice, 
                    x_min, y_min, axis_x, axis_y, BoxSize, periodic, verbose=False)

            # project particle velocity*mass into a 2D maps
            VM = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(VM, pos_g_slice, Vg_slice*Mg_slice, Rg_slice, 
                               x_min, y_min,
                               axis_x, axis_y, BoxSize, periodic, verbose=False)

            # project gas particle mass into a 2D maps
            Mgas = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(Mgas, pos_g_slice, Mg_slice, Rg_slice, x_min, y_min,
                               axis_x, axis_y, BoxSize, periodic, verbose=False)

            # project dm particle mass into a 2D maps
            Mcdm = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(Mcdm, pos_c_slice, Mc_slice, Rc_slice, x_min, y_min,
                               axis_x, axis_y, BoxSize, periodic, verbose=False)

            # project star particle mass into a 2D maps
            Mstar = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(Mstar, pos_s_slice, Ms_slice, Rs_slice, x_min, y_min,
                               axis_x, axis_y, BoxSize, periodic, verbose=False)

            # project gas particle HI into a 2D maps
            HIgas = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(HIgas, pos_g_slice, HI_slice, Rg_slice, x_min, y_min,
                               axis_x, axis_y, BoxSize, periodic, verbose=False)

            # project electron density into a 2D maps
            negas = np.zeros((grid,grid), dtype=np.float64)
            MASL.voronoi_RT_2D(negas, pos_g_slice, ne_slice, Rg_slice, x_min, y_min,
                               axis_x, axis_y, BoxSize, periodic, verbose=False)

            maps_T[count]  = TM/Mgas
            maps_Z[count]  = ZM/Mgas
            maps_Vg[count] = VM/Mgas
            maps_P[count]  = PM/Mgas
            maps_Mg[count] = Mgas
            maps_Mc[count] = Mcdm
            maps_Ms[count] = Mstar
            maps_HI[count] = HIgas
            maps_ne[count] = negas
            count += 1

    print('%d maps'%count)
    print('%.3e < T [K]  < %.3e'%(np.min(maps_T),  np.max(maps_T)))
    print('%.3e <   Z    < %.3e'%(np.min(maps_Z),  np.max(maps_Z)))
    print('%.3e <  Mgas  < %.3e'%(np.min(maps_Mg), np.max(maps_Mg)))
    print('%.3e <  Mcdm  < %.3e'%(np.min(maps_Mc), np.max(maps_Mc)))
    print('%.3e <  Mstar < %.3e'%(np.min(maps_Ms), np.max(maps_Ms)))
    print('%.3e <  Vgas  < %.3e'%(np.min(maps_Vg), np.max(maps_Vg)))
    print('%.3e <   HI   < %.3e'%(np.min(maps_HI), np.max(maps_HI)))
    print('%.3e <   n_e  < %.3e'%(np.min(maps_ne), np.max(maps_ne)))
    print('%.3e <   P    < %.3e'%(np.min(maps_P),  np.max(maps_P)))

    fout1 = '%s_T.npy'%fout;      np.save(fout1, maps_T)
    fout2 = '%s_Z.npy'%fout;      np.save(fout2, maps_Z)
    fout3 = '%s_Mgas.npy'%fout;   np.save(fout3, maps_Mg)
    fout4 = '%s_Mstar.npy'%fout;  np.save(fout4, maps_Ms)
    fout5 = '%s_Vgas.npy'%fout;   np.save(fout5, maps_Vg)
    fout6 = '%s_HI.npy'%fout;     np.save(fout6, maps_HI)
    fout7 = '%s_Mcdm.npy'%fout;   np.save(fout7, maps_Mc)
    fout8 = '%s_ne.npy'%fout;     np.save(fout8, maps_ne)
    fout9 = '%s_P.npy'%fout;      np.save(fout9, maps_P)
    

    # save images
    #maps_total = np.log10(maps_total)
    #maps_total = torch.tensor(maps_total, dtype=torch.float32)
    #maps_total = maps_total.unsqueeze(1)
    #print(maps_total.shape)
    #vutils.save_image(maps_total, 'images_fiducial_0.png',
    #                  normalize=True, nrow=5)
